#include "PrefiM_bsp.h"
#include <stddef.h>
#include <stdio.h>
#include "nordic_common.h"
#include "nrf.h"
#include "nrf_gpio.h"
#include "nrf_drv_gpiote.h"
#include "nrf_soc.h"
#include "nrf_delay.h"
#include "globals.h"
#include "nrf_drv_pwm.h"
#include "nrf_log.h"
#include "Interrupts.h"
#include "nrf_drv_timer.h"
#include "nrf_drv_saadc.h"
#include "nrf_drv_ppi.h"

// ****************************************  Wired IO:s  ****************************************

uint8_t PMBSP_read_DIP(void)
{
	DIPstatus = 0;
	
	uint8_t build_timer;
	nrf_gpio_pin_pull_t in_pullup=NRF_GPIO_PIN_PULLUP;
	
	
		//All pins fir DIPSWITCH and TAMPER and MAGNET should be set to IN at init. When application start we do a readout. 
	
	
		nrf_gpio_cfg_input(PIN_DIP_1, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_2, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_3, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_4, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_5, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_6, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_7, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_8, in_pullup);

	if(!nrf_gpio_pin_read(PIN_DIP_1)) 
		DIPstatus |= 0x01; // Vanlig timer eller smart timer
	if(!nrf_gpio_pin_read(PIN_DIP_2))
		DIPstatus |= 0x02; // Timertid 1, vanligt timer: 00=av, 01 = 15min, 10=45min, 11 = 2h
	if(!nrf_gpio_pin_read(PIN_DIP_3)) 
		DIPstatus |= 0x04; // Timertid 2, softtimer: 00 = av, 01 = 5 min, 10=10min, 11 = 15min
	if(!nrf_gpio_pin_read(PIN_DIP_4)) 
		DIPstatus |= 0x08; // Kokfunktion
	if(!nrf_gpio_pin_read(PIN_DIP_5)) 
		DIPstatus |= 0x10; // Varningssignal
	if(!nrf_gpio_pin_read(PIN_DIP_6)) 
		DIPstatus |= 0x20; // Indikeringslampa
	if(!nrf_gpio_pin_read(PIN_DIP_7)) 
		DIPstatus |= 0x40; // Indikeringsljud
	if(!nrf_gpio_pin_read(PIN_DIP_8)) 
		DIPstatus |= 0x80; // Spisls / barnls
	
	#ifndef BATTERY_DRIVEN												//Smart timer only works if we have constant power
	if ( (DIPstatus & 0x01) ==  0x01)
	{
		DIP_Smart_timer = TRUE;
		DIP_Regular_timer = FALSE;
	}
	else 
	{
		DIP_Smart_timer = FALSE;
		DIP_Regular_timer = TRUE;
	}
	#else
		DIP_Smart_timer = FALSE;
		DIP_Regular_timer = TRUE;
	#endif
	
	
	build_timer = 0;
	
	if ((DIPstatus & 0x02) == 0x02)
		build_timer = build_timer + 2;

	if ((DIPstatus & 0x04) == 0x04)
		build_timer = build_timer + 1;

	switch(build_timer)
	{
		case 0: // Timer off
				DIP_Timer = 0;
				break;

		case 1:	// 5 min = 300 sec
				if(DIP_Smart_timer)
					DIP_Timer = 300;	// 5 min = 300 sec
				if(DIP_Regular_timer)
					DIP_Timer = 900;	// 15 min = 900 sec
				break;

		case 2:	
				if(DIP_Smart_timer)
					DIP_Timer = 600;	// 10 min = 600 sec
				if(DIP_Regular_timer)
					DIP_Timer = 2700;	// 45 min = 2700 sec
				break;

		case 3:	
				if(DIP_Smart_timer)
					DIP_Timer = 60;//900;	// 15 min = 900 sec
				if(DIP_Regular_timer)
					DIP_Timer = 7200;	// 2h min = 7200 sec
				break;
	}
	
	
	if ( (DIPstatus & 0x08) ==  0x08)
		DIP_Boiler = TRUE;
	else
		DIP_Boiler = FALSE;
	
	if ( (DIPstatus & 0x10) ==  0x10)
		DIP_Sound_warning = TRUE;
	else
		DIP_Sound_warning = FALSE;
	
	if ( (DIPstatus & 0x20) ==  0x20)
		DIP_LED_Indicator = TRUE;
	else
		DIP_LED_Indicator = FALSE;

	if ( (DIPstatus & 0x40) ==  0x40)
		DIP_use_Indicator = TRUE;
	else
		DIP_use_Indicator = FALSE;

	if ( (DIPstatus & 0x80) ==  0x80)
		DIP_MagnetControlled = TRUE;
	else
		DIP_MagnetControlled = FALSE;

	
/*	nrf_gpio_input_disconnect(PIN_DIP_1);
	nrf_gpio_input_disconnect(PIN_DIP_2);
	nrf_gpio_input_disconnect(PIN_DIP_3);
	nrf_gpio_input_disconnect(PIN_DIP_4);
	nrf_gpio_input_disconnect(PIN_DIP_5);
	nrf_gpio_input_disconnect(PIN_DIP_6);
	nrf_gpio_input_disconnect(PIN_DIP_7);
	nrf_gpio_input_disconnect(PIN_DIP_8);
*/	
	

	nrf_gpio_cfg_output(PIN_DIP_1);
	nrf_gpio_cfg_output(PIN_DIP_2);
	nrf_gpio_cfg_output(PIN_DIP_3);
	nrf_gpio_cfg_output(PIN_DIP_4);
	nrf_gpio_cfg_output(PIN_DIP_5);
	nrf_gpio_cfg_output(PIN_DIP_6);
	nrf_gpio_cfg_output(PIN_DIP_7);
	nrf_gpio_cfg_output(PIN_DIP_8);
	
	nrf_gpio_pin_clear(PIN_DIP_1);
	nrf_gpio_pin_clear(PIN_DIP_2);
	nrf_gpio_pin_clear(PIN_DIP_3);
	nrf_gpio_pin_clear(PIN_DIP_4);
	nrf_gpio_pin_clear(PIN_DIP_5);
	nrf_gpio_pin_clear(PIN_DIP_6);
	nrf_gpio_pin_clear(PIN_DIP_7);
	nrf_gpio_pin_clear(PIN_DIP_8);
	
	return DIPstatus;
}

uint8_t PMBSP_read_Prodtest(void)
{
	if (!nrf_drv_gpiote_in_is_set(PIN_PRODTEST))
		return FALSE;
	else
		return TRUE;
}

uint8_t PMBSP_read_Magnet(void)
{
	uint8_t result;
	
	nrf_gpio_pin_set(PIN_VCC_HALL);	//Power on HALL sensors befor reading
	nrf_delay_ms(10);	
	if(!nrf_gpio_pin_read(PIN_Magnet))
		result = TRUE;
	else
		result = FALSE;
	
	nrf_gpio_pin_clear(PIN_VCC_HALL);	//Power off HALL sensors after reading
	
	return result;
}

uint8_t PMBSP_read_Tamper(void)
{
	uint8_t result;
	
	nrf_gpio_pin_set(PIN_VCC_HALL);	//Power on HALL sensors befor reading
	nrf_delay_ms(10);	
	if(!nrf_gpio_pin_read(PIN_TAMPER))
		result = TRUE;
	else
		result = FALSE;
	
	nrf_gpio_pin_clear(PIN_VCC_HALL);	//Power off HALL sensors after reading
	
	return result;
}

uint8_t PMBSP_set_contactor(unsigned char timeout)
{
	nrf_drv_gpiote_out_clear(PIN_BreakerOut);
	
	return TRUE;
}

uint8_t PMBSP_release_contactor(unsigned char timeout)
{
	nrf_drv_gpiote_out_set(PIN_BreakerOut);

	return TRUE;
}

// ****************************************  Summer  ****************************************

void beep()
{
	uint32_t loop;
	nrf_gpio_pin_set(PIN_AudioSHDN);		//Enable speaker 
	nrf_delay_ms(2);
	//nrf_gpio_cfg_output(27);
	
	for(loop = 0; loop < 200; loop++)
	{
		
		nrf_gpio_pin_set(PIN_SPEAKER);
		nrf_delay_us(100);
		nrf_gpio_pin_clear(PIN_SPEAKER);
		nrf_delay_us(400);
	}	
	nrf_gpio_pin_clear(PIN_AudioSHDN);		//Enable speaker
}

static nrf_drv_pwm_t m_pwm0 = NRF_DRV_PWM_INSTANCE(1);

nrf_drv_pwm_config_t config0 =
{
		.output_pins =
		{
				PIN_SPEAKER,
				NRF_DRV_PWM_PIN_NOT_USED,  // channel 1
				NRF_DRV_PWM_PIN_NOT_USED,  // channel 2
				NRF_DRV_PWM_PIN_NOT_USED  // channel 3
		},
		.irq_priority = APP_IRQ_PRIORITY_LOWEST,
		.base_clock   = NRF_PWM_CLK_1MHz,
		.count_mode   = NRF_PWM_MODE_UP,
		.top_value    = 250, // 4kHz
		.load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
		.step_mode    = NRF_PWM_STEP_AUTO
};
		
static nrf_pwm_values_individual_t seq_values_pwm;
		
nrf_pwm_sequence_t const seq =
{
		.values.p_individual = &seq_values_pwm,
		.length          = NRF_PWM_VALUES_LENGTH(seq_values_pwm),
		.repeats         = 0,
		.end_delay       = 0
};

// DC frn 0 till 100%, frekvens i Hz
void PMBSP_update_sound(uint8_t duty_cycle, uint16_t frequency)
{
		uint16_t newvalue = 0;
	
		newvalue = (int) (1000000 / frequency);
	
		config0.top_value = newvalue;
    seq_values_pwm.channel_0 = (int) ( (newvalue * duty_cycle) / 100 );
	
		nrf_drv_pwm_uninit(&m_pwm0);
	  APP_ERROR_CHECK(nrf_drv_pwm_init(&m_pwm0, &config0, NULL));
//	  nrf_drv_pwm_sequence_update(&m_pwm0, 0, &seq);
	
		if(duty_cycle > 0)
		{
			nrf_gpio_pin_set(PIN_AudioSHDN);
		  (void)nrf_drv_pwm_simple_playback(&m_pwm0, &seq, 1, NRF_DRV_PWM_FLAG_LOOP);
		}
		else
		{
			nrf_drv_pwm_uninit(&m_pwm0);
		   nrf_gpio_cfg_output(PIN_SPEAKER);
			nrf_gpio_pin_clear(PIN_SPEAKER);
			nrf_gpio_pin_clear(PIN_AudioSHDN);
		}
	
    //nrf_drv_pwm_simple_playback(&m_pwm0, &seq, 1, 0);
	NRF_LOG_INFO("Update Sound! D/C = %d, Freq = %d,  PWMvalue = %d, Topvalue = %d\r\n", duty_cycle, frequency, seq_values_pwm.channel_0, newvalue);
}
void stop_sound(void)
{
		nrf_drv_pwm_uninit(&m_pwm0);

}

// ****************************************  RGB LED  ****************************************

//static nrf_drv_pwm_t m_LED_pwm = NRF_DRV_PWM_INSTANCE(2);

nrf_drv_pwm_config_t LED_pwm_config =
{
		.output_pins =
		{
				PIN_LedR | NRF_DRV_PWM_PIN_INVERTED, //
				PIN_LedG | NRF_DRV_PWM_PIN_INVERTED, // channel 1
				PIN_LedB | NRF_DRV_PWM_PIN_INVERTED, // channel 2
		},
		.irq_priority = APP_IRQ_PRIORITY_LOWEST,
		.base_clock   = NRF_PWM_CLK_1MHz,
		.count_mode   = NRF_PWM_MODE_UP,
		.top_value    = 250, // 4kHz
		.load_mode    = NRF_PWM_LOAD_INDIVIDUAL,
		.step_mode    = NRF_PWM_STEP_AUTO
};
static nrf_drv_pwm_t m_LED_pwm = NRF_DRV_PWM_INSTANCE(0);
static nrf_pwm_values_individual_t LED_seq_values_pwm;
		
nrf_pwm_sequence_t const LED_seq =
{
		.values.p_individual = &LED_seq_values_pwm,
		.length          = NRF_PWM_VALUES_LENGTH(LED_seq_values_pwm),
		.repeats         = 0,
		.end_delay       = 0
};

void PMBSP_set_leds(uint8_t R_PWM, uint8_t G_PWM, uint8_t B_PWM, uint8_t blink_status)
{
		//return;
		
		LED_pwm_config.top_value = 100;
    LED_seq_values_pwm.channel_0 = 0xFF - R_PWM;
    LED_seq_values_pwm.channel_1 = 0xFF - G_PWM;
    LED_seq_values_pwm.channel_2 = 0xFF - B_PWM;


	
		nrf_drv_pwm_uninit(&m_LED_pwm);
	
		if((R_PWM + G_PWM + B_PWM) > 0)
		{
			APP_ERROR_CHECK(nrf_drv_pwm_init(&m_LED_pwm, &LED_pwm_config, NULL));
	
			(void)nrf_drv_pwm_simple_playback(&m_LED_pwm, &LED_seq, 1, NRF_DRV_PWM_FLAG_LOOP);
		}
		else
		{
		//nrf_gpio_cfg_output(22); // Elda inte upp summern
			nrf_gpio_pin_clear(PIN_LedR);
  		nrf_gpio_pin_clear(PIN_LedG);
			nrf_gpio_pin_clear(PIN_LedB);

		}
		//nrf_drv_pwm_uninit(&m_LED_pwm);		
		/*				nrf_gpio_pin_set(PIN_LedR);
				nrf_gpio_pin_clear(PIN_LedG);
				nrf_gpio_pin_clear(PIN_LedB);
		*/
			
}

// ****************************************  Internal Sensors  ****************************************

int8_t PMBSP_readtemperature(void)
{
		int32_t raw_temp;
		int8_t temp;
		sd_temp_get(&raw_temp);
		temp = raw_temp >> 2;  // divide by 4

		return temp;	
}

// ****************************************  Initialization  ****************************************

void PMBSP_IO_init(void)
{
		ret_code_t err_code;
		//nrf_drv_gpiote_in_config_t in_config;
	//nrf_gpio_pin_input_t in_config;
	
		//in_config = NRF_GPIO_PIN_INPUT_CONNECT;


		nrf_gpio_pin_pull_t in_pullup=NRF_GPIO_PIN_PULLUP;
	
		//All pins fir DIPSWITCH and TAMPER and MAGNET should be set to IN at init. When application start we do a readout. 
	
		nrf_gpio_cfg_input(PIN_TAMPER, in_pullup);
		nrf_gpio_cfg_input(PIN_Magnet,in_pullup);
	
	/*
		nrf_gpio_cfg_input(PIN_DIP_1, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_2, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_3, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_4, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_5, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_6, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_7, in_pullup);
		nrf_gpio_cfg_input(PIN_DIP_8, in_pullup);
	*/		
		nrf_gpio_cfg_output(PIN_DIP_1);
		nrf_gpio_cfg_output(PIN_DIP_2);
		nrf_gpio_cfg_output(PIN_DIP_3);
		nrf_gpio_cfg_output(PIN_DIP_4);
		nrf_gpio_cfg_output(PIN_DIP_5);
		nrf_gpio_cfg_output(PIN_DIP_6);
		nrf_gpio_cfg_output(PIN_DIP_7);
		nrf_gpio_cfg_output(PIN_DIP_8);
		
		nrf_gpio_pin_clear(PIN_DIP_1);
		nrf_gpio_pin_clear(PIN_DIP_2);
		nrf_gpio_pin_clear(PIN_DIP_3);
		nrf_gpio_pin_clear(PIN_DIP_4);
		nrf_gpio_pin_clear(PIN_DIP_5);
		nrf_gpio_pin_clear(PIN_DIP_6);
		nrf_gpio_pin_clear(PIN_DIP_7);
		nrf_gpio_pin_clear(PIN_DIP_8);
		
		nrf_gpio_cfg_output(PIN_LedR);
		nrf_gpio_cfg_output(PIN_LedG);
		nrf_gpio_cfg_output(PIN_LedB);
		
		nrf_gpio_cfg_output(PIN_SPEAKER);
		nrf_gpio_cfg_output(PIN_AudioSHDN);
		
		nrf_gpio_pin_clear(PIN_SPEAKER);
		nrf_gpio_pin_clear(PIN_AudioSHDN);
		

		nrf_gpio_pin_clear(PIN_LedR);
		nrf_gpio_pin_clear(PIN_LedG);
		nrf_gpio_pin_clear(PIN_LedB);


		nrf_gpio_cfg_output(PIN_BreakerOut); 
		nrf_gpio_cfg_output(PIN_LoadDetectOut); 
		
		nrf_gpio_pin_set(PIN_BreakerOut);
		nrf_gpio_pin_clear(PIN_LoadDetectOut);
		
		/******************************************************************************
		//New pins for battery version, Prototype Candidate 1.3 and forward
		******************************************************************************/
		nrf_gpio_cfg_input(PIN_MODESELECT, in_pullup);			//Pin which will be read once at startup to decide if it is battery driven or not.
		
		nrf_gpio_cfg_output(PIN_VCC_I2CEN);
		nrf_gpio_cfg_output(PIN_VCC_HALL);
		nrf_gpio_pin_set(PIN_VCC_I2CEN);
		nrf_gpio_pin_set(PIN_VCC_HALL);
		
	
	/*
	  err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);
		
		nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_TOGGLE(false);
		in_config.pull = NRF_GPIO_PIN_PULLUP;
	
		err_code = nrf_drv_gpiote_in_init(PIN_TAMPER, &in_config, TAMPER_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_TAMPER, true);

		err_code = nrf_drv_gpiote_in_init(PIN_Magnet, &in_config, MAGNET_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_Magnet, true);
	
		err_code = nrf_drv_gpiote_in_init(PIN_DIP_1, &in_config, DIP_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_DIP_1, true);

		err_code = nrf_drv_gpiote_in_init(PIN_DIP_2, &in_config, DIP_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_DIP_2, true);

		err_code = nrf_drv_gpiote_in_init(PIN_DIP_3, &in_config, DIP_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_DIP_3, true);

		err_code = nrf_drv_gpiote_in_init(PIN_DIP_4, &in_config, DIP_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_DIP_4, true);

		err_code = nrf_drv_gpiote_in_init(PIN_DIP_5, &in_config, DIP_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_DIP_5, true);

		err_code = nrf_drv_gpiote_in_init(PIN_DIP_6, &in_config, DIP_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_DIP_6, true);

		err_code = nrf_drv_gpiote_in_init(PIN_DIP_7, &in_config, DIP_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_DIP_7, true);

		err_code = nrf_drv_gpiote_in_init(PIN_DIP_8, &in_config, DIP_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(PIN_DIP_8, true);
*/
//		nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_ SENSE_TOGGLE(false);
		//in_config.pull = NRF_GPIO_PIN_PULLUP;

		nrf_gpio_cfg_output(PIN_LedR);
		nrf_gpio_cfg_output(PIN_LedG);
		nrf_gpio_cfg_output(PIN_LedB);

		nrf_gpio_pin_clear(PIN_LedR);
		nrf_gpio_pin_clear(PIN_LedG);
		nrf_gpio_pin_clear(PIN_LedB);


		nrf_gpio_cfg_output(PIN_BreakerOut); 
		nrf_gpio_cfg_output(PIN_LoadDetectOut); 
		
		nrf_gpio_pin_set(PIN_BreakerOut);
		nrf_gpio_pin_clear(PIN_LoadDetectOut);
}

// ****************************************  AD Converter  ****************************************

#define SAMPLES_IN_BUFFER 2

volatile uint8_t state = 1;

static const nrf_drv_timer_t m_timer = NRF_DRV_TIMER_INSTANCE(3);
static nrf_saadc_value_t     m_buffer_pool[2][SAMPLES_IN_BUFFER];
static nrf_ppi_channel_t     m_ppi_channel;
//static uint32_t              m_adc_evt_counter;

void timer_handler(nrf_timer_event_t event_type, void* p_context)
{
 beep();
}

void saadc_sampling_event_init(void)
{
    ret_code_t err_code;

    err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);

    nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    timer_cfg.bit_width = NRF_TIMER_BIT_WIDTH_32;
    err_code = nrf_drv_timer_init(&m_timer, &timer_cfg, timer_handler);
    APP_ERROR_CHECK(err_code);

    /* setup m_timer for compare event every 1000ms */
    uint32_t ticks = nrf_drv_timer_ms_to_ticks(&m_timer, 1000);
    nrf_drv_timer_extended_compare(&m_timer,
                                   NRF_TIMER_CC_CHANNEL3,
                                   ticks,
                                   NRF_TIMER_SHORT_COMPARE3_CLEAR_MASK,
                                   false);
	
    nrf_drv_timer_enable(&m_timer);

    uint32_t timer_compare_event_addr = nrf_drv_timer_compare_event_address_get(&m_timer, NRF_TIMER_CC_CHANNEL3);
    uint32_t saadc_sample_task_addr   = nrf_drv_saadc_sample_task_get();

    /* setup ppi channel so that timer compare event is triggering sample task in SAADC */
    err_code = nrf_drv_ppi_channel_alloc(&m_ppi_channel);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_ppi_channel_assign(m_ppi_channel, timer_compare_event_addr, saadc_sample_task_addr);
    APP_ERROR_CHECK(err_code);
}


void saadc_sampling_event_enable(void)
{
    ret_code_t err_code = nrf_drv_ppi_channel_enable(m_ppi_channel);
    APP_ERROR_CHECK(err_code);
}

void saadc_callback(nrf_drv_saadc_evt_t const * p_event)
{
    if (p_event->type == NRF_DRV_SAADC_EVT_DONE)
    {
        ret_code_t err_code;

        err_code = nrf_drv_saadc_buffer_convert(p_event->data.done.p_buffer, SAMPLES_IN_BUFFER);
        APP_ERROR_CHECK(err_code);

/*        int i;
        NRF_LOG_INFO("ADC event number: %d\r\n", (int)m_adc_evt_counter);

        for (i = 0; i < SAMPLES_IN_BUFFER; i++)
        {
            NRF_LOG_INFO("%d\r\n", p_event->data.done.p_buffer[i]);
        }
        m_adc_evt_counter++;
*/    }
		
		float voltage;		

		voltage = (p_event->data.done.p_buffer[0]) * (( 6 * 7.8 * 0.6) / 0x3FFF);
		NRF_LOG_INFO("Supply Voltage:" NRF_LOG_FLOAT_MARKER "V\r\n", NRF_LOG_FLOAT(voltage));

//		ble_sghs_vext_update(m_conn_handle, &m_sghs, voltage);

		voltage = (p_event->data.done.p_buffer[1]) * ( 6 * 0.6 / 0x3FFF);
		NRF_LOG_INFO("Internal VCC:" NRF_LOG_FLOAT_MARKER "V\r\n", NRF_LOG_FLOAT(voltage));
		
//		ble_sghs_vint_update(m_conn_handle, &m_sghs, voltage);
		
//		nrf_drv_saadc_calibrate_offset();		

//beep();
}

void PMBSP_saadc_init(void)
{
    ret_code_t err_code;
	
    nrf_drv_saadc_config_t saadc_config = NRF_DRV_SAADC_DEFAULT_CONFIG;
    saadc_config.resolution = NRF_SAADC_RESOLUTION_14BIT;
	  saadc_config.oversample = NRF_SAADC_OVERSAMPLE_256X;
		
	
    nrf_saadc_channel_config_t channel_0_config = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_AIN6);
    channel_0_config.gain = NRF_SAADC_GAIN1_6;
    channel_0_config.reference = NRF_SAADC_REFERENCE_INTERNAL;
	  channel_0_config.burst = NRF_SAADC_BURST_ENABLED;
		channel_0_config.acq_time = NRF_SAADC_ACQTIME_40US;

	
    nrf_saadc_channel_config_t channel_1_config = NRF_DRV_SAADC_DEFAULT_CHANNEL_CONFIG_SE(NRF_SAADC_INPUT_VDD);
    channel_1_config.gain = NRF_SAADC_GAIN1_6;
    channel_1_config.reference = NRF_SAADC_REFERENCE_INTERNAL;
	  channel_1_config.burst = NRF_SAADC_BURST_ENABLED;
		channel_1_config.acq_time = NRF_SAADC_ACQTIME_40US;

    err_code = nrf_drv_saadc_init(&saadc_config, saadc_callback);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_saadc_channel_init(0, &channel_0_config);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_saadc_channel_init(1, &channel_1_config);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_saadc_buffer_convert(m_buffer_pool[0],SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(err_code);   
    err_code = nrf_drv_saadc_buffer_convert(m_buffer_pool[1],SAMPLES_IN_BUFFER);
    APP_ERROR_CHECK(err_code);
}

#define MAX_REC_COUNT      3     /**< Maximum records count. */

uint8_t m_ndef_msg_buf[256];

#include "nfc_t2t_lib.h"
#include "nfc_ndef_msg.h"
#include "nfc_text_rec.h"
#include "nfc_launchapp_rec.h"
#include "nfc_launchapp_msg.h"
#include "BLE_middleware.h"

// ****************************************  NFC  ****************************************

/**
 * @brief Callback function for handling NFC events.
 */
static void nfc_callback(void * p_context, nfc_t2t_event_t event, const uint8_t * p_data, size_t data_length)
{
    (void)p_context;

    switch (event)
    {
        case NFC_T2T_EVENT_FIELD_ON:
						beep();
						PMBLE_advertising_start(0);
            //bsp_board_led_on(BSP_BOARD_LED_0);
            break;
        case NFC_T2T_EVENT_FIELD_OFF:
						beep();
            //bsp_board_led_off(BSP_BOARD_LED_0);
            break;
        default:
            break;
    }
}

///**
// * @brief Function for creating a record.
// */
//static void t1_record_add(nfc_ndef_msg_desc_t * p_ndef_msg_desc)
//{
//    /** @snippet [NFC text usage_1] */
//    uint32_t             err_code;
//    static const uint8_t payload[] =
//                  {'P', 'r', 'e', 'f', 'i', ' ', 'M'};
//    static const uint8_t code[] = {'S', 'V'};

//    NFC_NDEF_TEXT_RECORD_DESC_DEF(t1_text_rec,
//                                  UTF_8,
//                                  code,
//                                  sizeof(code),
//                                  payload,
//                                  sizeof(payload));
//   /** @snippet [NFC text usage_1] */

//    err_code = nfc_ndef_msg_record_add(p_ndef_msg_desc,
//                                       &NFC_NDEF_TEXT_RECORD_DESC(t1_text_rec));
//    APP_ERROR_CHECK(err_code);

//}


///**
// * @brief Function for creating a record.
// */
//static void t2_record_add(nfc_ndef_msg_desc_t * p_ndef_msg_desc)
//{
//    uint32_t             err_code;
//    static const uint8_t no_payload[] =
//                          {'C', 'a', 'b', 'i', 'n', 'o', 'v', 'a', ' ', 'I', 'n', 't', 'e', 'r', 'n', 'a', 't', 'i', 'o', 'n', 'a', 'l', ' ', 'A', 'B'};
//    static const uint8_t no_code[] = {'S', 'V'};

//    NFC_NDEF_TEXT_RECORD_DESC_DEF(no_text_rec,
//                                  UTF_8,
//                                  no_code,
//                                  sizeof(no_code),
//                                  no_payload,
//                                  sizeof(no_payload));

//    err_code = nfc_ndef_msg_record_add(p_ndef_msg_desc,
//                                       &NFC_NDEF_TEXT_RECORD_DESC(no_text_rec));
//    APP_ERROR_CHECK(err_code);
//}

/**
 * @brief Function for encoding the welcome message.
 */
//static void text_msg_encode(uint8_t * p_buffer, uint32_t * p_len)
//{
//    NFC_NDEF_MSG_DEF(welcome_msg, MAX_REC_COUNT);

//    t1_record_add(&NFC_NDEF_MSG(welcome_msg));
//    t2_record_add(&NFC_NDEF_MSG(welcome_msg));

//    /** @snippet [NFC text usage_2] */
//    uint32_t err_code = nfc_ndef_msg_encode(&NFC_NDEF_MSG(welcome_msg),
//                                            p_buffer,
//                                            p_len);
//    APP_ERROR_CHECK(err_code);
//    /** @snippet [NFC text usage_2] */
//}

void PMBSP_NFC_init(void)
{
	uint32_t len;
	uint32_t err_code;

	// Set up NFC 
	err_code = nfc_t2t_setup(nfc_callback, NULL);
	APP_ERROR_CHECK(err_code);

	// Provide information about available buffer size to encoding function.
	len = sizeof(m_ndef_msg_buf);
	
	// Encode text message 
//    text_msg_encode(m_ndef_msg_buf, &len);

	// Encode launchapp message into buffer 
	err_code = nfc_launchapp_msg_encode(m_android_package_name,
																			sizeof(m_android_package_name),
																			NULL,
																			0,
																			m_ndef_msg_buf,
																			&len);
	APP_ERROR_CHECK(err_code);

	// Set created message as the NFC payload
	err_code = nfc_t2t_payload_set(m_ndef_msg_buf, len);
	APP_ERROR_CHECK(err_code);

	// Start sensing NFC field
	err_code = nfc_t2t_emulation_start();
	APP_ERROR_CHECK(err_code);
}

