
#ifndef GLOBALS_H__
#define GLOBALS_H__

#include <stdint.h>
#include <stdbool.h>

#include "app_twi.h"
#include "ble_sghs.h"
#include "ble_sgis.h"
#include "ble_sgss.h"
#include "ble_sgds.h"
#include "ble_bas.h"
#include "ble_dis.h"
#include "nrf_drv_pwm.h"

#define POWER_SENSE
#define BATTERY_DRIVEN
//#define POWERED_BY_CABLE
#define WIRELESS
#define DEBUG_MODE

#define TRUE 1
#define FALSE 0

#define DEVICE_NAME                      "Prefi M"                               /**< Name of device. Will be included in the advertising data. */
#define MANUFACTURER_NAME                "Cabinova International AB"                      /**< Manufacturer. Will be passed to Device Information Service. */
#define HARDWARE_REVISION								 "Alpha 1.0"
#define FIRMWARE_REVISION								 "0.7 beta"
#define SERIAL													 "0000001"

#define APP_ADV_INTERVAL                 300                                        /**< The advertising interval (in units of 0.625 ms. This value corresponds to 187.5 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS       3000                                        /**< The advertising timeout in units of seconds. */

#define CONN_CFG_TAG                     1                                          /**< A tag that refers to the BLE stack configuration we set with @ref sd_ble_cfg_set. Default tag is @ref BLE_CONN_CFG_TAG_DEFAULT. */

#define BATTERY_LEVEL_MEAS_INTERVAL      APP_TIMER_TICKS(2000)                      /**< Battery level measurement interval (ticks). */
#define MIN_BATTERY_LEVEL                81                                         /**< Minimum simulated battery level. */
#define MAX_BATTERY_LEVEL                100                                        /**< Maximum simulated 7battery level. */
#define BATTERY_LEVEL_INCREMENT          1                                          /**< Increment between each simulated battery level measurement. */

#define MIN_CONN_INTERVAL                MSEC_TO_UNITS(800, UNIT_1_25_MS)     //MSEC_TO_UNITS(40, UNIT_1_25_MS)           /**< Minimum acceptable connection interval (0.4 seconds). */ //400
#define MAX_CONN_INTERVAL                MSEC_TO_UNITS(1300, UNIT_1_25_MS)     //MSEC_TO_UNITS(65, UNIT_1_25_MS)           /**< Maximum acceptable connection interval (0.65 second). */ // 650
#define SLAVE_LATENCY                    0                                          /**< Slave latency. */
#define CONN_SUP_TIMEOUT                 MSEC_TO_UNITS(4000, UNIT_10_MS)            /**< Connection supervisory timeout (4 seconds). */

#define FIRST_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(5000)                      /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY    APP_TIMER_TICKS(30000)                     /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT     3                                          /**< Number of attempts before giving up the connection parameter negotiation. */

#define SEC_PARAM_BOND                   1                                          /**< Perform bonding. */
#define SEC_PARAM_MITM                   0                                          /**< Man In The Middle protection not required. */
#define SEC_PARAM_LESC                   0                                          /**< LE Secure Connections not enabled. */
#define SEC_PARAM_KEYPRESS               0                                          /**< Keypress notifications not enabled. */
#define SEC_PARAM_IO_CAPABILITIES        BLE_GAP_IO_CAPS_NONE                       /**< No I/O capabilities. */
#define SEC_PARAM_OOB                    0                                          /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE           7                                          /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE           16                                         /**< Maximum encryption key size. */

#define DEAD_BEEF                        0xDEADBEEF                                 /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define APP_FEATURE_NOT_SUPPORTED        BLE_GATT_STATUS_ATTERR_APP_BEGIN + 2       /**< Reply when unsupported features are requested. */


extern const app_twi_t m_app_twi;

#define Pixel 1024
#define NUMBER_OF_ROWS 32
#define PIX_PER_ROW 32
extern signed short SensorImage[NUMBER_OF_ROWS][PIX_PER_ROW];	
extern signed short SensorImage_Hist_1s[NUMBER_OF_ROWS][PIX_PER_ROW];	

#define runmodeNORMAL 0
#define runmodeADVERTIZING 1
#define runmodeSGISTestInternalHardware 2
#define runmodeSGISTestMount 3
#define runmodeSGISTestUserInterface 4
#define runmodeSGISTestSensor 5

extern ble_sgis_t * p_sgis_main;

extern uint8_t runmode;
extern uint8_t DIP_Smart_timer;				   // TRUE = Smart timer
extern uint8_t DIP_Regular_timer;				// TRUE = Regular timer
extern uint32_t DIP_Timer;								// TRUE = Regular timer
extern uint8_t DIP_Boiler; // TRUE = Boiler function on
extern uint8_t DIP_Sound_warning; // TRUE = Sound signal on  
extern uint8_t DIP_LED_Indicator; // TRUE = LED indicators on
extern uint8_t DIP_use_Indicator; // TRUE = Summer useage indicator on
extern uint8_t DIP_MagnetControlled; // TRUE = Magnetkontrollerad brytare

extern uint16_t m_conn_handle; /**< Handle of the current connection. */
extern uint16_t m_conn_handle_peripheral_0;
extern ble_bas_t m_bas;                                   /**< Structure used to identify the battery service. */
extern ble_sghs_t m_sghs;					 								          /**< Main structures used by the Stove Guard Hardware service. */
extern ble_sgis_t m_sgis;					 								          /**< Main structures used by the Stove Guard Installation test service. */
extern ble_sgss_t m_sgss;					 								          /**< Main structures used by the Stove Guard Security service. */
extern ble_sgds_t m_sgds;					 								          /**< Main structures used by the Stove Guard Detector service. */

extern char BLE_connected_flag;

// Global timers
extern volatile uint8_t timer_magnetkey_running;
extern volatile uint32_t runtime_magnetkey;

extern volatile uint8_t timer_alarm_delay_running;
extern volatile uint32_t runtime_alarm_delay;

extern volatile uint8_t timer_movement_delay_running;
extern volatile uint32_t runtime_movement_delay;

extern volatile uint8_t timer_stove_running;
extern volatile uint32_t runtime_stove;

extern volatile uint8_t timer_sgisTest_running;
extern volatile uint32_t runtime_sgisTest;

extern volatile uint8_t timer_kontaktor_running;
extern volatile uint32_t runtime_kontaktor;

extern volatile uint8_t timer_advertising_running;
extern volatile uint32_t runtime_advertising;

extern unsigned char timer_rest_running;	
extern unsigned volatile int runtime_rest;



//**********************************************************************************
// Definition of different times for different timers
//**********************************************************************************

#define TIMER_MAGNETKEY_TIMEOUT 45    //7200 // 2 hours timeout forgotten magnetkey
#define TIMER_ALARM_DELAY_TIMEOUT 75 // 1 minut delay if you have moved a pan at the hotplate...Someone is present
#define TIMER_MOVEMENT_DELAY_TIMEOUT 60 // 20s when someone is moving something at the stove
#define TIMER_SGIS_TEST_MOUNT_DELAY_TIMEOUT 10 // 10s to perform SGIS test nr 1 and 2
#define TIMER_SGIS_TEST_USERINTERFACE_DELAY_TIMEOUT 30 // 30s to perform SGIS test nr 3, io test diods, summer, breaker, magnet key
#define TIMER_SGIS_TEST_SENSOR_DELAY_TIMEOUT 300 // 5 minuts to perform SGIS test nr 4, heat zone test
#define TIMER_ADVERTISING_TIMEOUT 30 // 5 minuts to perform SGIS test nr 4, heat zone test
#define TIMER_HEAT_MEASUREMENT_TIMEOUT 1	// We will read every second.

#define TEMP_NORMAL 0			// Inga konstiga derivator eller niver detekterade
#define TEMP_LOW_FAST_RISING 1		// 
#define TEMP_HIGH_FAST_RISING 2		//
#define TEMP_HIGH_FAST_DROPPING 3	//
#define TEMP_HIGHALARM 4	//
#define TEMP_STOVE_ON 5
#define TEMP_OVER_BOILING 6
#define	MOVEMENT_DETECTED 7

#define AMBIENT_TEMPERATURE_MAX_LIMIT 50

#define TL_CUT_POWER 160  //300
#define TL_LARM_SOUND 150	//270
#define TL_LARM_VISUAL 130	//200				//Temperatur som skall resultera i varningssignal
#define TL_OVER_BOILING 80 //110
#define TL_STOVE_ON 40									// minskar den s att den ven tar kokande vatten med lock  45
#define TL_BOILING_VISUAL_LARM_COMP 50		//TL_LARM_VISUAL-TL_OVER_BOILING
#define TL_BOILING_SOUND_LARM_COMP 73				//TL_LARM_SOUND-TL_OVER_BOILING +3
#define TL_BOILING_CUT_COMP	86							//TL_CUT_POWER-TL_OVER_BOILING + 6

//Konstant som stter toleransen p storleken p ett AOI som skall underskas. Anvnds i checkForRapidTemperature
#define AREA_SIZE_TOLERANCE 2

//**********************************************************************************
//TF=Temperature Feature. Konstanter som talar om fr MainStateMachine hur temperaturen ser ut
//**********************************************************************************
#define TF_CUT_POWER 5
#define TF_SECOND_LARM_LEVEL 4
#define TF_FIRST_LARM_LEVEL 3
#define TF_OVER_BOILING 2
#define TF_STOVE_ON 1
#define TF_STOVE_OFF 0

//Konstanter fr att styra niver fr att vi skall tolka det som en flytt av kastrull
#define TEMPERATURE_DERIVATE_TRESHHOLD 15
#define TEMPERATURE_DERIVATE_TRESHHOLD_BATTERY 50			//This treshhold is valid when we use batteri and only read temperature one every 15 second
#define NUMBER_OF_PIXELS_WITH_HIGH_DERIVATE_IN_AOI_TRESHHOLD 4
#define NUMBER_OF_PIXELS_WITH_HIGH_DERIVATE_TRESHHOLD 20

#define NEGATIVE_DERIVATE_TRESHHOLD 0	//Level for which we see temperature decrease
#define LOCATION_TOLERANCE 4			//how much can the position of the hotest spot alter.

#define TEMPERATURE_DIFFERENCE_TRESHHOLD 150							//The limit for which we say that we have movement in the thermal image. Temperature difference between two following images
#define AVERAGE_TEMPERATURE_DIFFERENCE_TRESHHOLD 2000



//**************************************Powersave*************************************************************************************************
extern uint8_t blockToRead;
extern char sensorReadoutMode;
#define SENSOR_READOUT_POWER_SAVING_MODE 0
#define SENSOR_READOUT_NORMAL_MODE 1

//**************************************Accelerometer*************************************************************************************************
/*#define X_MIN -1200	//-20
#define X_MAX -1000 //20
#define Z_MIN -100 //-20
#define Z_MAX 1000 //20
#define Y_MIN -100 //-1035
#define Y_MAX 100 //-1025
*/
#define X_MIN -1200
#define X_MAX -1000 //20
#define Z_MIN -50 //-20
#define Z_MAX 50 //20
#define Y_MIN -50 //-1035
#define Y_MAX 50 //-1025

//Variables for orientazion is found below, ***Variables that will be sent over BLE*****

//**************************************Accelerometer*************************************************************************************************/

extern uint8_t SOMEONE_PRESENT_FLAG;
extern uint8_t MOVEMENT_FLAG;
extern uint8_t MOVEMENT_ON_STOVE;
extern uint8_t TEMPERATURE_STATUS;
extern uint8_t SENSESTATE_OUTPUT;

//**************************************Sounds*************************************************************************************************/
#ifdef BATTERY_DRIVEN

//extern char sleep_mode;
#define SLEEP_MODE_ONE 0									//No sleep
#define SLEEP_MODE_NORMAL 1									//Sleep for 30s
#define SLEEP_MODE_THREE 2									//Sleep for 15s

#define FPU_EXCEPTION_MASK               0x0000009F                      //!< FPU exception mask used to clear exceptions in FPSCR register.
#define soundIntervalShutdownTime 240
#define soundIntervalShutdownTempterature 240
#define soundIntervalShutdownMagnet 240
#define soundIntervalPRE_REST 240
#define soundIntervalREST_DELAY 240
#define soundIntervalShutdownAmbientTemp 240

#define flashIntervalStoveOn 30

#else
#define soundIntervalShutdownTime 10
#define soundIntervalShutdownTempterature 20
#define soundIntervalShutdownMagnet 20
#define soundIntervalPRE_REST 20
#define soundIntervalREST_DELAY 20
#define soundIntervalShutdownAmbientTemp 15

#endif
#define FPU_EXCEPTION_MASK               0x0000009F
extern int whenToMakeSoundIndication;				//Variable to handle when to make sound in different states in mainStateMachine, indications.c
extern int whenToFlash;											//variable that handles when to flash
//**************************************MAINSTATEMACHINE*************************************************************************************************/
extern uint8_t MAGNETSTATUS;
extern uint8_t DIPstatus;
extern uint8_t warningLevelSound;
extern uint8_t warningLevelVisual;
extern uint8_t cutPowerLevel;
extern unsigned char mainstate;				// Main State Machine

#define STARTUP 1
#define STOVE_OFF 2
#define STOVE_ON 3
#define TEMPERATURE_WARNING_LEVEL_1 4
#define REST 5
#define CHECK_MAGNET 6
#define PRE_STOVE_OFF 7
#define FIRE 8
#define REST_CHECKMEASURE 9
#define REST_STARTMEASURE 10
#define REST_DELAY 11
#define PRE_REST 12
#define DOUBLECHECK_OFF 13
#define SHUTDOWN_TIME 14
#define SHUTDOWN_TEMP 15
#define SHUTDOWN_MAGNET 16
#define TEMPERATURE_WARNING_LEVEL_2 17
#define SHUTDOWN_AMBIENT_TEMP 18
#define ALARM_DELAY 19
#define LINK_FAILED 20

//**************************************POWER SENSE*************************************************************************************************/
#define TIMER_PRE_TIME_DELAY 60 // 60 sekunders paus innan vi slr p strmmen igen efter att anvndningstiden gtt ut
#define TIMER_PRE_TEMP_DELAY 60 // 60 sekunders paus innan vi slr p strmmen igen efter att vi har haft en verhettning fr att lta spisen svalna, samt att ev. verhettningsskydd i spisen fr tid att terstllas
#define TIMER_PULSE_WAITTIMEOUT 30 // 30 sekunder vntar vi efter att en pulserande last:s senaste puls borde ha kommit innan vi anser att vi skall terg till vilolge (i praktiken ge en ny brukstid)
#define TIMER_RESTART_DELAY 1 // 1 sekund fr en induktionshll p sig att starta

#define TIMER_KONTAKTOR_MAXPAUSTIME 180

#define POWER_LIMIT_STOVE_ON 250							//Level for when we say that the stove is on or not
extern unsigned volatile char LOAD_SENSE_PIN;	//This IO was used for load detection in Prefi 2.3 and we use the same algoritm in Prefi M so we need it to be named the same
extern unsigned int KONTAKTOR_WaitTime; // 0 -> 255 sekunders paus mellan belastningskollarna
extern unsigned int PREREST_WaitTime; // 0 -> 255 sekunders paus innan vi brjar kolla belastning
extern unsigned volatile char PowerMeasureLoadOn;
extern unsigned volatile char load_sense_counter;
extern unsigned volatile char PowerMeasurePeriod;
extern unsigned volatile char PowerMeasureStatus;
extern unsigned volatile char load_is_on;
extern  unsigned volatile int load_hist_time_off;
extern  unsigned volatile int load_sense_timer;
extern  unsigned volatile char load_off_valid;
extern unsigned volatile int load_hist_time_on;
extern unsigned int load_sense_waittimer;
extern unsigned volatile char load_on_valid;

extern unsigned char last_load_type;
extern unsigned char load_type;

extern unsigned char loadstate;

extern unsigned volatile char BELASTNING_STATUS;

extern unsigned int MAXRUNTIME;


extern uint16_t activePowerConsumtionBreaker;
extern uint16_t reactivePowerConsumtionBreaker;
extern char powerMode;
#define POWER_MODE_1s 1
#define POWER_MODE_15s 15
#define POWER_MODE_30s 30

#define LS_STARTUP 1
#define LS_NOLOAD 2
#define LS_FIRSTON 3
#define LS_FIRSTOFF 4
#define LS_WAITFIRSTLOAD 5
#define LS_LOADON 6
#define LS_LOADOFF 7
#define LS_FIRE 8

#define LOAD_UNIDENT 0
#define LOAD_STOVE 1	//Vanlig jrnspis
#define LOAD_GEN_PULSE 2	//Generell pulserande last

#define LOAD_NONE 0
#define LOAD_ON 1
#define LOAD_WAIT 2




//**************************************Variables that will be sent over BLE*************************************************************************************************/
#define PULL_BREAKER 0x01
#define RELEASE_BREAKER 0x00
extern signed short TaSensor;
extern int16_t acc_x;
extern int16_t acc_y;
extern int16_t acc_z;

extern short hotestTemperature;

extern uint8_t powerConsumtionBreaker[4];
extern int numberOfTimerBreaks;
extern int numberOfOverheatBreaks;


//**************************************Temperatures*************************************************************************************************/

extern char resultOfTemperatureAnalysis;



typedef struct
{
	char inUse;
	short xMin;
	short xMax;
	short yMin;
	short yMax;
	short hotestTemperature;
	short xPos;
	short yPos;
	int averageTemperature;
	int size;
	short highPositiveDerivate;
	short negativeDerivate;
} locationStruct;
extern locationStruct areasOfInterest[32]; 
extern locationStruct areasOfInterest_Hist_1s[32];
extern locationStruct hotestTemperatureHistogram_10s[10];
extern char whenToSaveHotestTemperatureCounter;


extern char movementOnStoveFlag;
//extern int minimumOfHumanPixels; 
// LED Colour
#define ALL_OFF	0
#define RED			1
#define GREEN		2
#define BLUE		3
#define MAGENTA	4
#define YELLOW	5
#define CYAN		6
#define WHITE		7

// LED Blink
#define SOLID				1
#define SLOW_BLINK	2
#define FAST_BLINK	3

// Summer
#define OFF				0
#define INDICATE 	1

/*Ticks before change duty cycle of each LED*/
#define TICKS_BEFORE_CHANGE_0   500
#define TICKS_BEFORE_CHANGE_1   400

#endif

