#include <stdint.h>
#include "ble_sghs.h"
#include "ble_sgis.h"
#include "ble_sgss.h"
#include "ble_sgds.h"

#include "nrf_log.h"
#include "PrefiM_bsp.h"
#include "nrf_delay.h"
#include "nrf_drv_gpiote.h"
#include "MMA8652FC.h"
#include "m_ApplicationInterface.h"
#include "HTPA32x32.h"
#include "m_SensorProcessing.h"
#include "m_ApplicationInit.h"
#include "softdevice_handler.h"
#include "app_timer.h"

void sghs_led_write_handler(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t R_PWM, uint8_t G_PWM, uint8_t B_PWM, uint8_t blink_status)
{
		NRF_LOG_INFO("LED status characteristic written, new value: R=%d, G=%d, B=%d, Blink=%d\r\n",R_PWM, G_PWM, B_PWM, blink_status);
	
		PMBSP_set_leds(R_PWM, G_PWM, B_PWM, blink_status);
		//PMAIF_LED_Set(RED,SOLID);
	
		nrf_delay_ms(1000);
}

void sghs_sound_write_handler(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t frequency, uint8_t volume, uint8_t pattern)
{
	//beep();
	
	// Frequency 0x00 => 1kHz, 0xFF => 10kHz
	// Volume 0x00 => Quiet, 0xFF = Max Vol
	
	// Max vol fs vid en DC p 40%
	uint8_t duty_cycle;
	uint16_t freq;
	
	duty_cycle = (uint8_t) (volume / (255/40) );

	float factor;
	
	factor = (float) frequency / 255;
	
#define minfreq  1000
#define maxfreq  6000
	
	freq =  (uint16_t) ( minfreq + ( factor * (maxfreq - minfreq) ) );
			
	NRF_LOG_INFO("Sound characteristic written: Frequency = %d, Volume = %d, Pattern = %d =========> D/C = %d, Ljudfrekvens = %d\r\n", frequency, volume, pattern, duty_cycle, freq);
	PMBSP_update_sound(duty_cycle, freq);

}


void sghs_bstat_write_handler(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t bstat_state)
{
		NRF_LOG_INFO("Breaker status characteristic written, new value: %d\r\n", bstat_state);

	if(bstat_state==0x00)
		//nrf_gpio_pin_set(PIN_BreakerOut);
		PMAIF_DisableStove();			
	else
		//nrf_gpio_pin_clear(PIN_BreakerOut);
		PMAIF_EnableStove();			
	
}

void sghs_lstat_write_handler(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t lstat_state)
{
			NRF_LOG_INFO("Load status characteristic written, new value: %d\r\n", lstat_state);
}

void sghs_prodtest_write_handler(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t pstat_state)
{
			NRF_LOG_INFO("Production Test characteristic written, new value: %d\r\n", pstat_state);
}

// *** SGIS ***
void sgis_command_write_handler(uint16_t conn_handle, ble_sgis_t * p_sgis, uint8_t command)
{
	//bool runflag = true;
	
//	uint16_t timeCounter = 0;
	NRF_LOG_INFO("Installation command: 0x%x\r\n",command);
	
	switch(command)
	{
		case 0x01:
				//Tell main() that we are in teststate for mounting test
				runmode = runmodeSGISTestInternalHardware;
				p_sgis_main=p_sgis;
			break;

		case 0x02:
				NRF_LOG_INFO("Start mount verification test\r\n");
			
				//Tell main() that we are in teststate for mounting test
				runmode = runmodeSGISTestMount;
				p_sgis_main=p_sgis;
					
			break;

		case 0x03:
				NRF_LOG_INFO("Start user interface test\r\n");
		
				//Tell main() that we are in teststate for userinterface test
				runmode = runmodeSGISTestUserInterface;
		
				//PMBSP_set_leds(0xFF,0xFF,0x00,0xFF);
				/*PMAIF_LED_Set(WHITE,SOLID);
				beep();		
				nrf_delay_ms(100);
				beep();		
				nrf_delay_ms(100);
				beep();		
				nrf_delay_ms(100);
				
				//Wait until magent places
				while(PMBSP_read_Magnet()==false);
				PMAIF_LED_Set(GREEN,SOLID);
				PMAIF_DisableStove();	
				nrf_delay_ms(500);
				PMAIF_LED_Set(BLUE,SOLID);
				PMAIF_EnableStove();	
				nrf_delay_ms(500);
				PMAIF_LED_Set(YELLOW,SOLID);
				PMAIF_DisableStove();	
				nrf_delay_ms(500);
				PMAIF_SUMMER_Set(1);
				nrf_delay_ms(100);			

				while(PMBSP_read_Magnet()==true);		//wiat for magnet to be removed
				PMAIF_LED_Set(GREEN,SOLID);
				nrf_delay_ms(500);
				
				ble_sgis_result_update(m_conn_handle, p_sgis, 0x03);*/
					
		break;

		case 0x04:
				NRF_LOG_INFO("Start heat source recognition test\r\n");
					//Tell main() that we are in teststate for sensor test
				runmode = runmodeSGISTestSensor;
		
				/*PMAIF_LED_Set(MAGENTA,SOLID);
				beep();	
				nrf_delay_ms(100);
				beep();		
				nrf_delay_ms(100);
				beep();		
				nrf_delay_ms(100);
				beep();		
				nrf_delay_ms(100);		
		
				
				
				HTPA32x32_readCompleteSensor();
				while((checkForTestHeatingZones()==false)&&(runflag == true))
				{
							HTPA32x32_readCompleteSensor();
							timeCounter++;
							if(timeCounter > 600)
								runflag = false;
		
				}
				if(checkForTestHeatingZones()==TRUE)
				{
					ble_sgis_result_update(m_conn_handle, p_sgis, 0x04);
					PMAIF_LED_Set(GREEN,SOLID);
					nrf_delay_ms(1000);
				}
				else
				{
					PMAIF_LED_Set(RED,SOLID);
					nrf_delay_ms(1000);
					ble_sgis_result_update(m_conn_handle, p_sgis, 0x14);
				}
				PMBSP_set_leds(0, 0, 0, 0);*/
			break;
		
		default:
				NRF_LOG_INFO("Not supported test case requested!\r\n");
				ble_sgis_result_update(m_conn_handle, p_sgis, 0x00);

			break;
	}
}

// *** SGSS ***
void sgss_key_write_handler(uint16_t conn_handle, ble_sgss_t * p_sgis, uint8_t command_state)
{
		NRF_LOG_INFO("Key recieved: 0x%x Seed set to the same!\r\n",command_state);	
	
}

// *** SGDS ***
void sgds_sptr_write_handler(uint16_t conn_handle, ble_sgds_t * p_sgds, uint8_t command_state)
{
		NRF_LOG_INFO("Matrix pointer recieved: 0x%x\r\n",command_state);		
}


