#include "globals.h"
#include "app_timer.h"
#include "low_power_pwm.h"
#include "Interrupts.h"
#include "PrefiM_bsp.h"
#include "nrf_delay.h"
#include "led_softblink.h"
#include "m_ApplicationInterface.h"

APP_TIMER_DEF(m_1Hz_timer_id);                        /**< 1Hz Softtimer. */

APP_TIMER_DEF(m_10Hz_timer_id);                        /**< 10Hz Softtimer. */

APP_TIMER_DEF(m_30sec_timer_id);                        /**< 30sec Softtimer. */

APP_TIMER_DEF(m_15sec_timer_id);                        /**< 30sec Softtimer. */


static void sec30_timeout_handler(void * p_context)
{
		if(timer_alarm_delay_running)
			runtime_alarm_delay=runtime_alarm_delay+30;	
		if(timer_magnetkey_running)
			runtime_magnetkey = runtime_magnetkey+30;
		if(timer_stove_running)
			runtime_stove = runtime_stove+30;
}
static void sec15_timeout_handler(void * p_context)
{
		if(timer_alarm_delay_running)
			runtime_alarm_delay=runtime_alarm_delay+15;	
		if(timer_magnetkey_running)
			runtime_magnetkey = runtime_magnetkey+15;
		if(timer_stove_running)
			runtime_stove = runtime_stove+15;	
		load_sense_timer = load_sense_timer+150;
}
static void oneHz_timeout_handler(void * p_context)
{
	UNUSED_PARAMETER(p_context);

	//printf("\r\n-- App timer timeout --\r\n");
	
	if(timer_magnetkey_running)
		runtime_magnetkey++;

	if(timer_alarm_delay_running)
		runtime_alarm_delay++;
	
	if(timer_stove_running)
		runtime_stove++;		
		

	if(timer_sgisTest_running)
		runtime_sgisTest++;	
	if(timer_kontaktor_running)
		runtime_kontaktor++;	
	if(timer_rest_running)
		runtime_rest++;
	if(timer_advertising_running)
		runtime_advertising++;

	load_sense_timer = load_sense_timer+10;
	//printf("\r\n Runtime Magnetkey = %u\r\n",runtime_magnetkey);
}

static void tenHz_timeout_handler(void * p_context)
{
	UNUSED_PARAMETER(p_context);

	//------------------------------- Hantera kontaktortillslag fr separat initierad belastningsmtning p 0.1-sekundintervall

// Denna del gr det mjligt att med konstant tidsprecision dra kontaktorn fr att kolla om objektets belastning r till
// Detta initieras genom att lgga in ett lmpligt vrde i PowerMeasurePeriod, vilket r en timeout fr hur lnge kontaktorn
// maximalt skall vara till.


		if (PowerMeasurePeriod > 0)				// En belastningsmtning har initierats
		{
			PowerMeasureStatus = TRUE;			// Indikera att vi hller p och mter belastning
												// Belastning skall mtas under s kort tid som mjligt
												// fr att frhindra uppvrmning

			//CONTACTOR_OUT_PIN = 0;				// Dra kontaktor utan tidskontroll
			PMAIF_EnableStove(); 
				PMAIF_LED_Set(YELLOW,SOLID);
				nrf_delay_ms(100);
				PMAIF_LED_Set(ALL_OFF,SOLID);
				nrf_delay_ms(500);
				PMAIF_LED_Set(YELLOW,SOLID);
				nrf_delay_ms(100);
				PMAIF_LED_Set(ALL_OFF,SOLID);
				nrf_delay_ms(500);
			PowerMeasurePeriod--;

			if (load_sense_counter > 0)			// Vi har haft belastning under tiden den kontrollerade mtningen pgtt
			{
				PowerMeasureLoadOn = TRUE;		// -> Indikera detta!
				PowerMeasurePeriod = 0;			// ... och avbryt mtningen nsta interrupttriggning (om 0.1 sek)
			}

		}
		else									// PowerMeasurePeriod = 0 == ingen pgende mtning.
		{
			if (PowerMeasureStatus == TRUE)		// PMP = TRUE => Vi har _precis nyss_ avslutat en tidsstyrd belastningsmtning,
			{									// antingen genom att PowerMeasurePeriod lpt ut (=LOAD_NONE kvarstr)
												// eller genom att PowerMeasurePeriod satts till noll pga av att last detekterats.
												// (=LOAD_ON)

				PowerMeasureStatus = FALSE;		// Indikera att vi slutat mta belastning

				if (PowerMeasureLoadOn == TRUE)
				{
					//CONTACTOR_OUT_PIN = 1;			// Slpp kontaktor utan tidskontroll ifall vi hade belastning
					PMAIF_DisableStove(); //Cut power
				}
			}
		}

//------------------------------- Hantera lastdetektering

		if (LOAD_SENSE_PIN == 1)				// Kolla om lastavknningen r statiskt lg
		{										// (eller rkar vara (pulsat) lg just nu)
			load_sense_counter++;				// som ett extra komplement till den interruptdrivna kningen av load_sense_counter
		}

// Vi har haft belastning de senaste 0.1 sek

		if(load_sense_counter > 0)
		{

			// Hantera pulsmtningen frst
			if (load_is_on == FALSE)	// Belastningen har slagits till nyligen
			{				
				load_hist_time_off = load_sense_timer;		// Spara timerns vrde som frra "frn"-tiden
				load_off_valid = TRUE;						// Indikera att OFF-tiden r OK att dra slutsatser utifrn
				load_sense_timer = 0;				// Nollstll timern fr att brja rkna "till"-tid				
			}

			load_is_on = TRUE;			// Indikera att vi har belastning till nsta avknning
	
			// Hantera resten

			load_sense_waittimer = 0;				// Se till att timern startar p 0 nr belastningen slpper

			if (PowerMeasurePeriod > 0)				// En belastningsmtning pgick!
			{										// Vi har dessutom detekterat en belastning
													// Vi behver inte vnta hela PowerMeasurePeriod
													// utan kan redan nu tala om att belastningen r till
													// (vilket grs p raderna ovan)

				PowerMeasurePeriod = 0;				// Och sl av belastningsmtningen
			}
		}

// Vi har inte haft belastning de senaste 0.1 sek

		else										
		{
			// Hantera pulsmtningen frst
			if (load_is_on == TRUE)	// Belastning har slagits frn nyligen
			{
				load_hist_time_on = load_sense_timer;		// Spara timerns vrde som frra "till"-tiden
				load_on_valid = TRUE;						// Indikera att ON-tiden r OK att dra slutsatser utifrn
				load_sense_timer = 0;				// Nollstll timern fr att brja rkna "frn"-tid				
			}
															
			load_is_on = FALSE;			// Indikera att vi inte har belastning till nsta avknning
		}

// Hantera timers

		// ---> ka timern
		if (load_sense_timer < 65400)			// Om vi inte uppntt maxgrns (65400 => 6540 sekunder = 109 minuter = 1h 49min)
			load_sense_timer++;		 			// ka timern oavsett om vi har belastning till eller frn
	

		load_sense_counter = 0;					// Nollstll belastningsindikering till nsta interrupt
												// Skulle lasten kvarst kas rknaren
												// av EXT0 interrupt eller i timerinterruptet
												// innan den anvnds igen.
	//printf("\r\n-- App timer timeout --\r\n");

	//printf("\r\n Runtime Magnetkey = %u\r\n",runtime_magnetkey);
}
///**@brief Function for the Timer initialization.
// *
// * @details Initializes the timer module. This creates and starts application timers.
// */
void PMAT_timers_init(void)
{
	ret_code_t err_code;

	// Init soft timers
	timer_magnetkey_running = FALSE;				//Timer magnetkey
	runtime_magnetkey = 0;
	
	timer_alarm_delay_running = FALSE;			//Timer high derivate
	runtime_alarm_delay = 0;		

	timer_stove_running	= FALSE;						//Timer when stove is on regarding to temperature
	runtime_stove = 0;
	
	
	timer_sgisTest_running	= FALSE;						//Timer when stove is on regarding to temperature
	runtime_sgisTest = 0;
	
	timer_kontaktor_running = FALSE;
	runtime_kontaktor = 0;
	
	timer_kontaktor_running = FALSE;
	runtime_advertising = 0;
	
	
	// Initialize timer module.
  err_code = app_timer_init();
  APP_ERROR_CHECK(err_code);


		err_code = app_timer_create(&m_30sec_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                sec30_timeout_handler);
    APP_ERROR_CHECK(err_code);
	
		err_code = app_timer_start(m_30sec_timer_id, APP_TIMER_TICKS(30000), NULL);
    APP_ERROR_CHECK(err_code);

		err_code = app_timer_create(&m_15sec_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                sec15_timeout_handler);
    APP_ERROR_CHECK(err_code);
	
		err_code = app_timer_start(m_15sec_timer_id, APP_TIMER_TICKS(15000), NULL);
    APP_ERROR_CHECK(err_code);


    err_code = app_timer_create(&m_1Hz_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                oneHz_timeout_handler);
    APP_ERROR_CHECK(err_code);
	
		err_code = app_timer_start(m_1Hz_timer_id, APP_TIMER_TICKS(1000), NULL);
    APP_ERROR_CHECK(err_code);
		
		err_code = app_timer_create(&m_10Hz_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                tenHz_timeout_handler);
    APP_ERROR_CHECK(err_code);
	
		err_code = app_timer_start(m_10Hz_timer_id, APP_TIMER_TICKS(100), NULL);
    APP_ERROR_CHECK(err_code);
		

}
//Stops all timers
void PMAT_stop_timers(void)
{
			ret_code_t err_code;
			err_code = app_timer_stop(m_1Hz_timer_id);
			err_code = app_timer_stop(m_10Hz_timer_id);
			err_code = app_timer_stop(m_15sec_timer_id);
			err_code = app_timer_stop(m_30sec_timer_id);
			APP_ERROR_CHECK(err_code);
}
//Stops short time timers, 0.1s, 1s, 15s timers.
void PMAT_stop_01s_1s_15s_timers(void)
{
			ret_code_t err_code;
			err_code = app_timer_stop(m_1Hz_timer_id);
			err_code = app_timer_stop(m_10Hz_timer_id);
			err_code = app_timer_stop(m_15sec_timer_id);
			//err_code = app_timer_stop(m_30sec_timer_id);
			APP_ERROR_CHECK(err_code);
}
//Stops timers, 0.1s, 1s, 30s timers.
void PMAT_stop_01s_1s_30s_timers(void)
{
			ret_code_t err_code;
			err_code = app_timer_stop(m_1Hz_timer_id);
			err_code = app_timer_stop(m_10Hz_timer_id);
			//err_code = app_timer_stop(m_15sec_timer_id);
			err_code = app_timer_stop(m_30sec_timer_id);
			APP_ERROR_CHECK(err_code);
}
//Stops timers, 0.1s, 1s, 30s timers.
void PMAT_stop_01s_15s_30s_timers(void)
{
			ret_code_t err_code;
			//err_code = app_timer_stop(m_1Hz_timer_id);
			err_code = app_timer_stop(m_10Hz_timer_id);
			err_code = app_timer_stop(m_15sec_timer_id);
			err_code = app_timer_stop(m_30sec_timer_id);
			APP_ERROR_CHECK(err_code);
}
void PMAT_start_timers(void)
{
	ret_code_t err_code;
	
	err_code = app_timer_start(m_10Hz_timer_id, APP_TIMER_TICKS(100), NULL);
  APP_ERROR_CHECK(err_code);
	
	err_code = app_timer_start(m_1Hz_timer_id, APP_TIMER_TICKS(1000), NULL);
  APP_ERROR_CHECK(err_code);
	
}
void PMAT_start_01s_15s_30s_timers(void)
{
			ret_code_t err_code;
	err_code = app_timer_start(m_10Hz_timer_id, APP_TIMER_TICKS(100), NULL);
  APP_ERROR_CHECK(err_code);
	
	//err_code = app_timer_start(m_15sec_timer_id, APP_TIMER_TICKS(1000), NULL);
  //APP_ERROR_CHECK(err_code);
	
	err_code = app_timer_start(m_30sec_timer_id, APP_TIMER_TICKS(30000), NULL);
  APP_ERROR_CHECK(err_code);
	
	err_code = app_timer_start(m_15sec_timer_id, APP_TIMER_TICKS(15000), NULL);
	APP_ERROR_CHECK(err_code);
}
void PMAT_start_01s_1s_30s_timers(void)
{
			ret_code_t err_code;
	err_code = app_timer_start(m_10Hz_timer_id, APP_TIMER_TICKS(100), NULL);
  APP_ERROR_CHECK(err_code);
	
	err_code = app_timer_start(m_1Hz_timer_id, APP_TIMER_TICKS(1000), NULL);
  APP_ERROR_CHECK(err_code);
	
	//err_code = app_timer_start(m_30sec_timer_id, APP_TIMER_TICKS(30000), NULL);
  //APP_ERROR_CHECK(err_code);
	
	err_code = app_timer_start(m_15sec_timer_id, APP_TIMER_TICKS(15000), NULL);
	APP_ERROR_CHECK(err_code);
}
void PMAT_start_30s_timers(void)
{
		ret_code_t err_code;
		err_code = app_timer_start(m_30sec_timer_id, APP_TIMER_TICKS(30000), NULL);
		APP_ERROR_CHECK(err_code);
	
	
}
void PMAT_start_15s_timers(void)
{
		ret_code_t err_code;
		err_code = app_timer_start(m_15sec_timer_id, APP_TIMER_TICKS(15000), NULL);
		APP_ERROR_CHECK(err_code);
	
	
}
void PMAT_start_01s_1s_15s_timers(void)
{
			ret_code_t err_code;
	err_code = app_timer_start(m_10Hz_timer_id, APP_TIMER_TICKS(100), NULL);
  APP_ERROR_CHECK(err_code);
	
	err_code = app_timer_start(m_15sec_timer_id, APP_TIMER_TICKS(15000), NULL);
  APP_ERROR_CHECK(err_code);
	
	err_code = app_timer_start(m_30sec_timer_id, APP_TIMER_TICKS(30000), NULL);
  APP_ERROR_CHECK(err_code);
	
	//err_code = app_timer_start(m_15sec_timer_id, APP_TIMER_TICKS(15000), NULL);
	//APP_ERROR_CHECK(err_code);
}

//    // Create timers.
//    err_code = app_timer_create(&m_battery_timer_id,
//                                APP_TIMER_MODE_REPEATED,
//                                battery_level_meas_timeout_handler);
//    APP_ERROR_CHECK(err_code);

///**@brief Function for starting application timers.
// */
//static void application_timers_start(void)
//{
//    ret_code_t err_code;

//    // Start application timers.
//    err_code = app_timer_start(m_battery_timer_id, BATTERY_LEVEL_MEAS_INTERVAL, NULL);
//    APP_ERROR_CHECK(err_code);
//}
