/**
 * Copyright (c) 2013 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#include "sdk_common.h"
#include "ble_sghs.h"
#include "ble_srv_common.h"
#include "PrefiM_bsp.h"
#include "globals.h"

/**@brief Function for handling the Write event.
 *
 * @param[in] p_sghs      LED Button Service structure.
 * @param[in] p_ble_evt  Event received from the BLE stack.
 */
static void sghs_on_write(ble_sghs_t * p_sghs, ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

    if ((p_evt_write->handle == p_sghs->led_char_handles.value_handle) &&
        (p_evt_write->len == 4) &&
        (p_sghs->led_write_handler != NULL))
    {
        p_sghs->led_write_handler(p_ble_evt->evt.gap_evt.conn_handle, p_sghs, p_evt_write->data[0], p_evt_write->data[1], p_evt_write->data[2], p_evt_write->data[3]);
    }
		
    if ((p_evt_write->handle == p_sghs->sound_char_handles.value_handle) &&
        (p_evt_write->len == 3) &&
        (p_sghs->sound_write_handler != NULL))
    {
        p_sghs->sound_write_handler(p_ble_evt->evt.gap_evt.conn_handle, p_sghs, p_evt_write->data[0], p_evt_write->data[1], p_evt_write->data[2]);
    }
		
    if ((p_evt_write->handle == p_sghs->bstat_char_handles.value_handle) &&
        (p_evt_write->len == 1) &&
        (p_sghs->bstat_write_handler != NULL))
    {
        p_sghs->bstat_write_handler(p_ble_evt->evt.gap_evt.conn_handle, p_sghs, p_evt_write->data[0]);
    }

    if ((p_evt_write->handle == p_sghs->lstat_char_handles.value_handle) &&
        (p_evt_write->len == 1) &&
        (p_sghs->lstat_write_handler != NULL))
    {
        p_sghs->lstat_write_handler(p_ble_evt->evt.gap_evt.conn_handle, p_sghs, p_evt_write->data[0]);
    }

    if ((p_evt_write->handle == p_sghs->prodtest_char_handles.value_handle) &&
        (p_evt_write->len == 1) &&
        (p_sghs->prodtest_write_handler != NULL))
    {
        p_sghs->prodtest_write_handler(p_ble_evt->evt.gap_evt.conn_handle, p_sghs, p_evt_write->data[0]);
    }
		
		
		if ((p_evt_write->handle == p_sghs->mbpower_char_handles.value_handle) &&
				(p_evt_write->len == 4) &&
				(p_sghs->mbpower_write_handler != NULL))
    {
        p_sghs->mbpower_write_handler(p_ble_evt->evt.gap_evt.conn_handle, p_sghs, p_evt_write->data[0], p_evt_write->data[1], p_evt_write->data[2], p_evt_write->data[3]);
    }
			
		if ((p_evt_write->handle == p_sghs->stoveguardstatus_char_handles.value_handle) &&
				(p_evt_write->len == 5) &&
				(p_sghs->stoveguardstatus_write_handler != NULL))
    {
        p_sghs->stoveguardstatus_write_handler(p_ble_evt->evt.gap_evt.conn_handle, p_sghs, p_evt_write->data[0], p_evt_write->data[1], p_evt_write->data[2], p_evt_write->data[3], p_evt_write->data[4]);
    }
		
}


void ble_sghs_on_ble_evt(ble_sghs_t * p_sghs, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GATTS_EVT_WRITE:
            sghs_on_write(p_sghs, p_ble_evt);
            break;

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for adding the LED Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t led_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
		uint8_t             initial_LED_status[4];

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_LED_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_LED_status[0] = 0;
		initial_LED_status[1] = 0;
		initial_LED_status[2] = 0;
		initial_LED_status[3] = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(initial_LED_status);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(initial_LED_status);
    attr_char_value.p_value      = initial_LED_status;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->led_char_handles);
}

/**@brief Function for adding the SOUND Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t sound_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
		uint8_t             initial_sound[3];

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_SOUND_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_sound[0] = 0;
		initial_sound[1] = 0;
		initial_sound[2] = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(initial_sound);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(initial_sound);
    attr_char_value.p_value      = initial_sound;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->sound_char_handles);
}

/**@brief Function for adding the BREAKER STATUS Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t bstat_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

		uint8_t             initial_bstat_status;
	
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
		char_md.char_props.notify = 1;
    char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_BSTAT_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));
		
		initial_bstat_status = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = &initial_bstat_status;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->bstat_char_handles);
}

/**@brief Function for adding the STOVEGUARD STATUS Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t stoveguardstatus_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

		uint8_t initial_stoveguard_status[5];
	
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
		char_md.char_props.notify = 1;
    char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_STOVEGUARDSTATUS_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));
		
		initial_stoveguard_status[0]=0;
		initial_stoveguard_status[1]=0;
		initial_stoveguard_status[2]=0;
		initial_stoveguard_status[3]=0;
		initial_stoveguard_status[4]=0;
		

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(initial_stoveguard_status);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(initial_stoveguard_status);
    attr_char_value.p_value      = initial_stoveguard_status;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->stoveguardstatus_char_handles);
}

/**@brief Function for adding the Multibreaker Power Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t mbpower_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

		uint8_t             initial_mbpower_status[4];
	
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_MBPOWER_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));
		
		initial_mbpower_status[0]=0;
		initial_mbpower_status[1]=0;
		initial_mbpower_status[2]=0;
		initial_mbpower_status[3]=0;
		

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(initial_mbpower_status);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(initial_mbpower_status);
    attr_char_value.p_value      = initial_mbpower_status;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->mbpower_char_handles);
}
/**@brief Function for adding the LOAD STATUS Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t lstat_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

		uint8_t             initial_lstat_status;
	
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_LSTAT_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_lstat_status = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = &initial_lstat_status;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->lstat_char_handles);
}

/**@brief Function for adding the LOAD STATUS Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t prodtest_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
		uint8_t             initial_ptest_status;

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_PRODTEST_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_ptest_status = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = &initial_ptest_status;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->prodtest_char_handles);
}

/**@brief Function for adding the TAMPER Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t tamper_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

		uint8_t             initial_tamper_status;
	
    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_TAMPER_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_tamper_status = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = &initial_tamper_status;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                               &char_md,
                                               &attr_char_value,
                                               &p_sghs->tamper_char_handles);
}

/**@brief Function for adding the DIP Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t dip_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
		uint8_t DIPstatus;

    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_DIP_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		DIPstatus = PMBSP_read_DIP();
	
    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = &DIPstatus;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                               &char_md,
                                               &attr_char_value,
                                               &p_sghs->dip_char_handles);
}

/**@brief Function for adding the MAGNET KEY Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t mkey_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    //ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

		uint8_t             initial_mkey_status;

	
    /*memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
	
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;
		*/
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.notify = 1;
		//char_md.char_props.write  = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL; //&cccd_md;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_MKEY_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_mkey_status = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = &initial_mkey_status;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                               &char_md,
                                               &attr_char_value,
                                               &p_sghs->mkey_char_handles);
}

/**@brief Function for adding the VINT Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t vint_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
		uint8_t             initial_vint_level[2];

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 0;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_VINT_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
//    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_vint_level[0] = 0;
		initial_vint_level[1] = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint16_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint16_t);
    attr_char_value.p_value      = initial_vint_level;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->vint_char_handles);
}

/**@brief Function for adding the VEXT Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t vext_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
		uint8_t             initial_vext_level[2];

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 0;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_VEXT_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
//    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_vext_level[0] = 0;
		initial_vext_level[1] = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint16_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint16_t);
    attr_char_value.p_value      = initial_vext_level;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sghs->vext_char_handles);
}

/**@brief Function for adding the MAGNET KEY Characteristic.
 *
 * @param[in] p_sghs      Stove Guard Service structure.
 * @param[in] p_sghs_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t cputemp_char_add(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;

    memset(&cccd_md, 0, sizeof(cccd_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_CPUTEMP_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = NULL;

    return sd_ble_gatts_characteristic_add(p_sghs->service_handle,
                                               &char_md,
                                               &attr_char_value,
                                               &p_sghs->cputemp_char_handles);
}

uint32_t ble_sghs_init(ble_sghs_t * p_sghs, const ble_sghs_init_t * p_sghs_init)
{
    uint32_t   err_code;
    ble_uuid_t ble_uuid;

    // Initialize service structure.
    p_sghs->led_write_handler = p_sghs_init->led_write_handler;
    p_sghs->sound_write_handler = p_sghs_init->sound_write_handler;
		p_sghs->bstat_write_handler = p_sghs_init->bstat_write_handler;
		p_sghs->lstat_write_handler = p_sghs_init->lstat_write_handler;
		p_sghs->prodtest_write_handler = p_sghs_init->prodtest_write_handler;
		p_sghs->mbpower_write_handler = p_sghs_init->mbpower_write_handler;

	
    // Add service.
    ble_uuid128_t base_uuid = {SGHS_UUID_BASE};
    err_code = sd_ble_uuid_vs_add(&base_uuid, &p_sghs->uuid_type);
    VERIFY_SUCCESS(err_code);

    ble_uuid.type = p_sghs->uuid_type;
    ble_uuid.uuid = SGHS_UUID_SERVICE;

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &p_sghs->service_handle);
    VERIFY_SUCCESS(err_code);

    // Add characteristics.
    err_code = tamper_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);

    err_code = led_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);

		err_code = dip_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);

		err_code = mkey_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);
		
		err_code = sound_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);

		err_code = bstat_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);

		err_code = lstat_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);

		err_code = vint_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);
		
		err_code = vext_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);
				
		err_code = prodtest_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);
		
		err_code = mbpower_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);
		
		err_code = stoveguardstatus_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);
		
		err_code = cputemp_char_add(p_sghs, p_sghs_init);
    VERIFY_SUCCESS(err_code);
		
    return NRF_SUCCESS;
}

uint32_t ble_sghs_on_tamper_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t button_state)
//uint32_t ble_sghs_on_tamper_change(ble_sghs_t * p_sghs, uint8_t button_state)
{
    ble_gatts_hvx_params_t params;
    uint16_t len = sizeof(button_state);

    memset(&params, 0, sizeof(params));
    params.type = BLE_GATT_HVX_NOTIFICATION;
    params.handle = p_sghs->tamper_char_handles.value_handle;
    params.p_data = &button_state;
    params.p_len = &len;

    return sd_ble_gatts_hvx(conn_handle, &params);
}

uint32_t ble_sghs_on_dip_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t dip_state)
{
    ble_gatts_hvx_params_t params;
    uint16_t len = sizeof(dip_state);

    memset(&params, 0, sizeof(params));
    params.type = BLE_GATT_HVX_NOTIFICATION;
    params.handle = p_sghs->dip_char_handles.value_handle;
    params.p_data = &dip_state;
    params.p_len = &len;

    return sd_ble_gatts_hvx(conn_handle, &params);
}

uint32_t ble_sghs_on_mkey_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t mkey_state)
{
	
		
    ble_gatts_hvx_params_t params;
    uint16_t len = sizeof(mkey_state);

    memset(&params, 0, sizeof(params));
    params.type = BLE_GATT_HVX_NOTIFICATION;
    params.handle = p_sghs->mkey_char_handles.value_handle;
    params.p_data = &mkey_state;
    params.p_len = &len;

    return sd_ble_gatts_hvx(conn_handle, &params);
}
uint32_t ble_sghs_on_bstat_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t breaker_state)
{
    ble_gatts_hvx_params_t params;
    uint16_t len = sizeof(breaker_state);

    memset(&params, 0, sizeof(params));
    params.type = BLE_GATT_HVX_NOTIFICATION;
    params.handle = p_sghs->bstat_char_handles.value_handle;
    params.p_data = &breaker_state;
    params.p_len = &len;

    return sd_ble_gatts_hvx(conn_handle, &params);
	
	
}
uint32_t ble_sghs_on_stoveguardstatus_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t main_state)
{
		uint8_t stoveguardStatusParameters[5];

		//First byte is depending on main_state only. If main_state = 15, SHUTDOWN_TEMP, or  = 18 SHUTDOWN_AMBIENT_TEMPERATURE than overheat is detected
		if((main_state == SHUTDOWN_TEMP)||(main_state == SHUTDOWN_AMBIENT_TEMP))
		{
			stoveguardStatusParameters[0]=0x01;			//Overheat detected. See BLE Profile Specification
			stoveguardStatusParameters[1]=main_state;
		}
		else
		{
			stoveguardStatusParameters[0]=0x00;			//Normal operation. See BLE Profile Specification
			stoveguardStatusParameters[1]=main_state;
			
		}
		
		//stoveguardStatusParameters[1]=0;
		//stoveguardStatusParameters[2]=0;
		stoveguardStatusParameters[3]=hotestTemperature;
		stoveguardStatusParameters[4]=powerMode;
		
		
    ble_gatts_hvx_params_t params;
    uint16_t len = sizeof(stoveguardStatusParameters);

    memset(&params, 0, sizeof(params));
    params.type = BLE_GATT_HVX_NOTIFICATION;
    params.handle = p_sghs->stoveguardstatus_char_handles.value_handle;
    params.p_data = stoveguardStatusParameters;
    params.p_len = &len;

    return sd_ble_gatts_hvx(conn_handle, &params);
	
	
}

uint32_t ble_sghs_on_cputemp_change(uint16_t conn_handle, ble_sghs_t * p_sghs, uint8_t temperature)
{
    ble_gatts_hvx_params_t params;
    uint16_t len = sizeof(temperature);

    memset(&params, 0, sizeof(params));
    params.type = BLE_GATT_HVX_NOTIFICATION;
    params.handle = p_sghs->cputemp_char_handles.value_handle;
    params.p_data = &temperature;
    params.p_len = &len;

    return sd_ble_gatts_hvx(conn_handle, &params);
}



uint32_t ble_sghs_vint_update(uint16_t conn_handle, ble_sghs_t * p_sghs, float voltage)
{
    if (p_sghs == NULL)
    {
        return NRF_ERROR_NULL;
    }

		
			uint16_t	vint_level;
			uint16_t	vint_len = sizeof(vint_level);
		
			vint_level = (uint16_t) ( 100 * voltage );
		
		
    uint32_t err_code = NRF_SUCCESS;
    ble_gatts_value_t gatts_value;

  //  if (battery_level != p_bas->battery_level_last)
  //  {
        // Initialize value struct.
        memset(&gatts_value, 0, sizeof(gatts_value));

        gatts_value.len     = vint_len;
        gatts_value.offset  = 0;
        gatts_value.p_value = (uint8_t*)&vint_level;

        // Update database.
        err_code = sd_ble_gatts_value_set(conn_handle,
                                          p_sghs->vint_char_handles.value_handle,
                                          &gatts_value);
        if (err_code == NRF_SUCCESS)
        {
            // Save new battery value.
  //          p_bas->battery_level_last = battery_level;
        }
        else
        {
            return err_code;
        }

        // Send value if connected and notifying.
    /*    if ((p_bas->conn_handle != BLE_CONN_HANDLE_INVALID) && p_bas->is_notification_supported)
        {
            ble_gatts_hvx_params_t hvx_params;

            memset(&hvx_params, 0, sizeof(hvx_params));

            hvx_params.handle = p_bas->battery_level_handles.value_handle;
            hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
            hvx_params.offset = gatts_value.offset;
            hvx_params.p_len  = &gatts_value.len;
            hvx_params.p_data = gatts_value.p_value;

            err_code = sd_ble_gatts_hvx(p_bas->conn_handle, &hvx_params);
        }
        else
        {
            err_code = NRF_ERROR_INVALID_STATE;
        }
				*/
  // }

    return err_code;
}

uint32_t ble_sghs_vext_update(uint16_t conn_handle, ble_sghs_t * p_sghs, float voltage)
{
    if (p_sghs == NULL)
    {
        return NRF_ERROR_NULL;
    }

		
			uint16_t	vext_level;
			uint16_t	vext_len = sizeof(vext_level);
		
			vext_level = (uint16_t) ( 100 * voltage );
		
		
    uint32_t err_code = NRF_SUCCESS;
    ble_gatts_value_t gatts_value;


        // Initialize value struct.
        memset(&gatts_value, 0, sizeof(gatts_value));

        gatts_value.len     = vext_len;
        gatts_value.offset  = 0;
        gatts_value.p_value = (uint8_t*)&vext_level;

        // Update database.
        err_code = sd_ble_gatts_value_set(conn_handle,
                                          p_sghs->vext_char_handles.value_handle,
                                          &gatts_value);
		
        if (err_code == NRF_SUCCESS)
        {

        }
        else
        {
            return err_code;
        }

    return err_code;
}

// 32x32 bytes = 1024 bytes
