/**
 * Copyright (c) 2013 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#include "sdk_common.h"
#include "ble_sgis.h"
#include "ble_srv_common.h"


/**@brief Function for handling the Write event.
 *
 * @param[in] p_sgis      LED Button Service structure.
 * @param[in] p_ble_evt  Event received from the BLE stack.
 */
static void sgis_on_write(ble_sgis_t * p_sgis, ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

    if ((p_evt_write->handle == p_sgis->command_char_handles.value_handle) &&
        (p_evt_write->len == 1) &&
        (p_sgis->command_write_handler != NULL))
    {
        p_sgis->command_write_handler(p_ble_evt->evt.gap_evt.conn_handle, p_sgis, p_evt_write->data[0]);
    }
}


void ble_sgis_on_ble_evt(ble_sgis_t * p_sgis, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GATTS_EVT_WRITE:
            sgis_on_write(p_sgis, p_ble_evt);
            break;

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for adding the COMMAND Characteristic.
 *
 * @param[in] p_sgis      Stove Guard Service structure.
 * @param[in] p_sgis_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t command_char_add(ble_sgis_t * p_sgis, const ble_sgis_init_t * p_sgis_init)
{
    ble_gatts_char_md_t char_md;
	//    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
		uint8_t             initial_command;

	 //   memset(&cccd_md, 0, sizeof(cccd_md));

 //   BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
 //   BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
 //   cccd_md.vloc = BLE_GATTS_VLOC_STACK;
	
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 1;
//	    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;// &cccd_md;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sgis->uuid_type;
    ble_uuid.uuid = SGIS_UUID_COMMAND_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_command = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = &initial_command;

    return sd_ble_gatts_characteristic_add(p_sgis->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sgis->command_char_handles);
}

/**@brief Function for adding the RESULT Characteristic.
 *
 * @param[in] p_sgis      Stove Guard Service structure.
 * @param[in] p_sgis_init Stove Guard Service initialization structure.
 *
 * @retval NRF_SUCCESS on success, else an error value from the SoftDevice
 */
static uint32_t result_char_add(ble_sgis_t * p_sgis, const ble_sgis_init_t * p_sgis_init)
{
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_uuid_t          ble_uuid;
    ble_gatts_attr_md_t attr_md;
	
		uint8_t             initial_result;

    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.read   = 1;
    char_md.char_props.write  = 0;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = NULL;
    char_md.p_sccd_md         = NULL;

    ble_uuid.type = p_sgis->uuid_type;
    ble_uuid.uuid = SGIS_UUID_RESULT_CHAR;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

		initial_result = 0;

    attr_char_value.p_uuid       = &ble_uuid;
    attr_char_value.p_attr_md    = &attr_md;
    attr_char_value.init_len     = sizeof(uint8_t);
    attr_char_value.init_offs    = 0;
    attr_char_value.max_len      = sizeof(uint8_t);
    attr_char_value.p_value      = &initial_result;

    return sd_ble_gatts_characteristic_add(p_sgis->service_handle,
                                           &char_md,
                                           &attr_char_value,
                                           &p_sgis->result_char_handles);
}



uint32_t ble_sgis_init(ble_sgis_t * p_sgis, const ble_sgis_init_t * p_sgis_init)
{
    uint32_t   err_code;
    ble_uuid_t ble_uuid;

    // Initialize service structure.
    p_sgis->command_write_handler = p_sgis_init->command_write_handler;

	
    // Add service.
    ble_uuid128_t base_uuid = {SGIS_UUID_BASE};
    err_code = sd_ble_uuid_vs_add(&base_uuid, &p_sgis->uuid_type);
    VERIFY_SUCCESS(err_code);

    ble_uuid.type = p_sgis->uuid_type;
    ble_uuid.uuid = SGIS_UUID_SERVICE;

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &p_sgis->service_handle);
    VERIFY_SUCCESS(err_code);

    // Add characteristics.
    err_code = command_char_add(p_sgis, p_sgis_init);
    VERIFY_SUCCESS(err_code);

    err_code = result_char_add(p_sgis, p_sgis_init);
    VERIFY_SUCCESS(err_code);

    return NRF_SUCCESS;
}


uint32_t ble_sgis_result_update(uint16_t conn_handle, ble_sgis_t * p_sgis, uint8_t result)
{
    if (p_sgis == NULL)
    {
        return NRF_ERROR_NULL;
    }

    uint32_t err_code = NRF_SUCCESS;
    ble_gatts_value_t gatts_value;

  //  if (battery_level != p_bas->battery_level_last)
  //  {
        // Initialize value struct.
        memset(&gatts_value, 0, sizeof(gatts_value));

        gatts_value.len     = sizeof(uint8_t);
        gatts_value.offset  = 0;
        gatts_value.p_value = &result;

        // Update database.
        err_code = sd_ble_gatts_value_set(conn_handle,
                                          p_sgis->result_char_handles.value_handle,
                                          &gatts_value);
        if (err_code == NRF_SUCCESS)
        {
            // Save new battery value.
  //          p_bas->battery_level_last = battery_level;
        }
        else
        {
            return err_code;
        }

        // Send value if connected and notifying.
    /*    if ((p_bas->conn_handle != BLE_CONN_HANDLE_INVALID) && p_bas->is_notification_supported)
        {
            ble_gatts_hvx_params_t hvx_params;

            memset(&hvx_params, 0, sizeof(hvx_params));

            hvx_params.handle = p_bas->battery_level_handles.value_handle;
            hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
            hvx_params.offset = gatts_value.offset;
            hvx_params.p_len  = &gatts_value.len;
            hvx_params.p_data = gatts_value.p_value;

            err_code = sd_ble_gatts_hvx(p_bas->conn_handle, &hvx_params);
        }
        else
        {
            err_code = NRF_ERROR_INVALID_STATE;
        }
				*/
  // }

    return err_code;
}

/*
uint32_t ble_sgis_on_dip_change(uint16_t conn_handle, ble_sgis_t * p_sgis, uint8_t dip_state)
{
    ble_gatts_hvx_params_t params;
    uint16_t len = sizeof(dip_state);

    memset(&params, 0, sizeof(params));
    params.type = BLE_GATT_HVX_NOTIFICATION;
    params.handle = p_sgis->dip_char_handles.value_handle;
    params.p_data = &dip_state;
    params.p_len = &len;

    return sd_ble_gatts_hvx(conn_handle, &params);
}
*/

// 32x32 bytes = 1024 bytes
