/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 * @defgroup nrf_twi_master_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Example Application main file.
 *
 * This file contains the source code for a sample application using TWI.
 *
 * @image html example_board_setup_a.jpg "Use board setup A for this example."
 */


#include <stdio.h>
#include "boards.h"
#include "app_util_platform.h"
#include "app_error.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "MMA8652FC.h"
#include "globals.h"
#include "m_ApplicationInterface.h"


#define NRF_LOG_MODULE_NAME "MMA8652"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"



/* Buffer for samples read from accelerometer */
//static uint8_t m_sample[10];
//static uint8_t m_sample;


// Buffer for data read from sensors.
#define BUFFER_SIZE  11
static uint8_t m_buffer[BUFFER_SIZE];

/* Indicates if operation on TWI has ended. */
static volatile bool MMA8652_xfer_done = false;

#if (BUFFER_SIZE < MMA8652_NUMBER_OF_REGISTERS)
    #error Buffer too small.
#endif

// Common callback function for all MMA8652 functions
static void read_mma8652_registers_cb(ret_code_t result, void * p_user_data)
{
    if (result != NRF_SUCCESS)
    {
        NRF_LOG_INFO("read_mma8652_registers_cb - error: %d\r\n", (int)result);
        return;
    }

//    NRF_LOG_INFO("MMA8652:\r\n");
//    NRF_LOG_HEXDUMP_INFO(m_buffer, MMA8652_NUMBER_OF_REGISTERS);
		
		MMA8652_xfer_done = true;
		
}

// Get current acceleration values
uint8_t mma8652_xout_reg_addr = MMA8652_REG_X_MSB;

void MMA8652_GetAccelerationXYZ()
{
	PMAIF_StartI2C();
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_xout_reg_addr, m_buffer, 6)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		//APP_ERROR_CHECK(app_twi_perform(&m_app_twi, transfers, 1, NULL));
		while (MMA8652_xfer_done == false);
		
		int16_t x = (int16_t)(((int16_t)m_buffer[0] << 8) | m_buffer[1]) / 16;
    int16_t y = (int16_t)(((int16_t)m_buffer[2] << 8) | m_buffer[3]) / 16;
    int16_t z = (int16_t)(((int16_t)m_buffer[4] << 8) | m_buffer[5]) / 16;
		
		NRF_LOG_INFO("Acceleration: X=%d, Y=%d, Z=%d\r\n", x,y,z);	
		
		PMAIF_StopI2C();
}		
// Set init configuration registers
/*static uint8_t init_config[] = { MMA8652_REG_CTRL_REG1, 0x01 };

app_twi_transfer_t MMA8652_init_device[MMA8652_INIT_TRANSFER_COUNT] =
{
    APP_TWI_WRITE(MMA8652_ADDR, init_config, sizeof(init_config), 0)
};*/
bool MMA8652_InHorizontalPosition()
{
	PMAIF_StartI2C();
	
	MMA8652_Sample();
	    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_xout_reg_addr, m_buffer, 6)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		
		acc_x = (int16_t)(((int16_t)m_buffer[0] << 8) | m_buffer[1]) / 16;
    acc_y = (int16_t)(((int16_t)m_buffer[2] << 8) | m_buffer[3]) / 16;
    acc_z = (int16_t)(((int16_t)m_buffer[4] << 8) | m_buffer[5]) / 16;
		
		NRF_LOG_INFO("Acceleration: X=%d, Y=%d, Z=%d\r\n", acc_x,acc_y,acc_z);	
		
		//printf("X Y Z: %d  %d %d",acc_x,acc_y,acc_z);
		//printf("\r\n -------------------------------------------------------\n");
		
		PMAIF_StopI2C();
		
		if((acc_y < Y_MAX)&(acc_y > Y_MIN)&(acc_z < Z_MAX)&(acc_z > Z_MIN)&(acc_x < X_MAX)&(acc_x > X_MIN))
			return true;
		else
			return false;
	
}
void MMA8652_Sample(void)
{
		static uint8_t init_config[] = { MMA8652_REG_CTRL_REG1, 0x01 };
		static app_twi_transfer_t MMA8652_init_device[MMA8652_INIT_TRANSFER_COUNT] =
		{
			APP_TWI_WRITE(MMA8652_ADDR, init_config, sizeof(init_config), 0)
		};
		
		 static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = MMA8652_init_device,
        .number_of_transfers = sizeof(MMA8652_init_device) / sizeof(MMA8652_init_device[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		//PMAIF_StopI2C();
}
void MMA8652_init_accelerometer()
{
		static uint8_t init_config[] = { MMA8652_REG_CTRL_REG1, 0x01 };
		static app_twi_transfer_t MMA8652_init_device[MMA8652_INIT_TRANSFER_COUNT] =
		{
			APP_TWI_WRITE(MMA8652_ADDR, init_config, sizeof(init_config), 0)
		};
		
		 static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = MMA8652_init_device,
        .number_of_transfers = sizeof(MMA8652_init_device) / sizeof(MMA8652_init_device[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		//PMAIF_StopI2C();
		
}
// Get current acceleration values
uint8_t mma8652_ctrl_Reg1 = MMA8652_REG_CTRL_REG1;
void MMA8652_ReadCTRLReg1()
{
	PMAIF_StartI2C();
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_ctrl_Reg1, m_buffer, 1)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		PMAIF_StopI2C();
		
	
}		
// Get CNTRL_REG2 data
uint8_t mma8652_ctrl_Reg2 = MMA8652_REG_CTRL_REG2;
void MMA8652_ReadCTRLReg2()
{
	PMAIF_StartI2C();
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_ctrl_Reg2, m_buffer, 1)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		PMAIF_StopI2C();
		
	
}		
// Get current acceleration values
uint8_t mma8652_ctrl_Reg3 = MMA8652_REG_CTRL_REG3;
void MMA8652_ReadCTRLReg3()
{
	PMAIF_StartI2C();
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_ctrl_Reg3, m_buffer, 1)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		PMAIF_StopI2C();
		
	
}		
// Get current acceleration values
uint8_t mma8652_ctrl_Reg4 = MMA8652_REG_CTRL_REG4;
void MMA8652_ReadCTRLReg4()
{
	PMAIF_StartI2C();
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_ctrl_Reg4, m_buffer, 1)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		PMAIF_StopI2C();
		
	
}		
// Get current acceleration values
uint8_t mma8652_ctrl_Reg5 = MMA8652_REG_CTRL_REG5;
void MMA8652_ReadCTRLReg5()
{
		PMAIF_StartI2C();
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_ctrl_Reg5, m_buffer, 1)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		PMAIF_StopI2C();
		
	
}		
// Read the SYSMOD register
uint8_t mma8652_SYSMOD = MMA8652_REG_SYSMOD;
void MMA865_ReadSYSMOD()
	{
		PMAIF_StartI2C();
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_SYSMOD, m_buffer, 1)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		PMAIF_StopI2C();
		
	
}		
	// Read the SYSMOD register
uint8_t mma8652_INTSOURCE = MMA8652_REG_INT_SOURCE;
void MMA865_ReadINTSOURCE()
	{
		PMAIF_StartI2C();
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_INTSOURCE, m_buffer, 1)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		PMAIF_StopI2C();
		
	
}		
// Verify device is on bus
static uint8_t mma8652_who_am_i = MMA8652_REG_WHO_AM_I;


bool MMA8652_verify_device()
{
	PMAIF_StartI2C();
	  static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_who_am_i, m_buffer, 1)
    };
		
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);

		if(m_buffer[0] == 0x4A)
			return true;
		else
			return false;
		PMAIF_StopI2C();
}

// Verify device is on bus
static uint8_t mma8652_status_register = 0x11;

void MMA8652_read_status_register()
{
	PMAIF_StartI2C();
		  static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_status_register, m_buffer, 1)
    };
		
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);
		
		PMAIF_StopI2C();
}

// Verify device is on bus
static uint8_t mma8652_system_mode_register = MMA8652_REG_SYSMOD;

void MMA8652_read_system_mode_register()
{
	PMAIF_StartI2C();
	  static app_twi_transfer_t const transfers[] =
    {
        MMA8652_READ(&mma8652_system_mode_register, m_buffer, 1)
				
    };
		
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma8652_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

		MMA8652_xfer_done = false;
    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
		while (MMA8652_xfer_done == false);

		/*if(m_buffer[0] == 0x4A)
			return true;
		else
			return false;*/
		PMAIF_StopI2C();
}

