/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include <stdint.h>
#include "app_twi.h"
#include "globals.h"

  
#ifndef MMA8652_H__
#define MMA8652_H__

#ifdef __cplusplus
extern "C" {
#endif


// 0x3A / 0x3B is the MMA8652's address, it contains
// R/W bit and "nrf_drv_twi" requires slave address without this bit, hence shifting.
#define MMA8652_ADDR      					  (0x3AU >> 1)		//  = 0x1DU

#define MMA8652_REG_STATUS						0x00
#define MMA8652_REG_X_MSB							0x01
#define MMA8652_REG_X_LSB							0x02
#define MMA8652_REG_Y_MSB							0x03
#define MMA8652_REG_Y_LSB							0x04
#define MMA8652_REG_Z_MSB							0x05
#define MMA8652_REG_Z_LSB							0x06

#define MMA8652_REG_F_SETUP						0x09
#define MMA8652_REG_TRIG_CFG					0x0A
#define MMA8652_REG_SYSMOD						0x0B
#define MMA8652_REG_INT_SOURCE				0x0C
#define MMA8652_REG_WHO_AM_I					0x0D
#define MMA8652_REG_XYZ_DATA_CFG			0x0E
#define MMA8652_REG_HP_FILTER_CUTOFF	0x0F
#define MMA8652_REG_PL_STATUS					0x10
#define MMA8652_REG_PL_CFG						0x11
#define MMA8652_REG_PL_COUNT					0x12
#define MMA8652_REG_PL_BF_ZCOMP				0x13
#define MMA8652_REG_P_L_THS_REG				0x14
#define MMA8652_REG_FF_MT_CFG					0x15
#define MMA8652_REG_FF_MT_SRC					0x16
#define MMA8652_REG_FF_MT_THS					0x17
#define MMA8652_REG_FF_MT_COUNT				0x18

#define MMA8652_REG_TRANSIENT_CFG			0x1D
#define MMA8652_REG_TRANSIENT_SRC			0x1E
#define MMA8652_REG_TRANSIENT_THS			0x1F
#define MMA8652_REG_TRANSIENT_COUNT		0x20
#define MMA8652_REG_PULSE_CFG					0x21
#define MMA8652_REG_PULSE_SRC					0x22
#define MMA8652_REG_PULSE_THSX				0x23
#define MMA8652_REG_PULSE_THSY				0x24
#define MMA8652_REG_PULSE_THSZ				0x25
#define MMA8652_REG_PULSE_TMLT				0x26
#define MMA8652_REG_PULSE_LTCY				0x27
#define MMA8652_REG_PULSE_WIND				0x28
#define MMA8652_REG_ASLP_COUNT				0x29
#define MMA8652_REG_CTRL_REG1					0x2A
#define MMA8652_REG_CTRL_REG2					0x2B
#define MMA8652_REG_CTRL_REG3					0x2C
#define MMA8652_REG_CTRL_REG4					0x2D
#define MMA8652_REG_CTRL_REG5					0x2E
#define MMA8652_REG_OFF_X							0x2F
#define MMA8652_REG_OFF_Y							0x30
#define MMA8652_REG_OFF_Z							0x31

#define MMA8652_NUMBER_OF_REGISTERS		6

#define MMA8652_READ(p_reg_addr, p_buffer, byte_cnt) \
    APP_TWI_WRITE(MMA8652_ADDR, p_reg_addr, 1,        APP_TWI_NO_STOP), \
    APP_TWI_READ (MMA8652_ADDR, p_buffer,   byte_cnt, 0)

#define MMA8652_INIT_TRANSFER_COUNT 1
extern app_twi_transfer_t MMA8652_init_device[MMA8652_INIT_TRANSFER_COUNT];

bool MMA8652_verify_device(void);
void MMA8652_GetAccelerationXYZ(void);
bool MMA8652_InHorizontalPosition(void);
void MMA8652_init_accelerometer(void);						//No init. We dont set any values, we just start the reading
void MMA8652_read_status_register(void);
void MMA8652_read_system_mode_register(void);
void MMA8652_ReadCTRLReg1(void);
void MMA8652_ReadCTRLReg2(void);
void MMA8652_ReadCTRLReg3(void);
void MMA8652_ReadCTRLReg4(void);
void MMA8652_ReadCTRLReg5(void);
void MMA8652_ReadSYSMOD(void);
void MMA8652_ReadINTSOURCE(void);
void MMA8652_Sample(void);
//void MMA8652_StartReadContinuously(void);	//Nop interupts at all. The accelerometer is just reading.
#ifdef __cplusplus
}
#endif

#endif // MMA8652_H__
