/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *	 list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *	 Semiconductor ASA integrated circuit in a product or a software update for
 *	 such product, must reproduce the above copyright notice, this list of
 *	 conditions and the following disclaimer in the documentation and/or other
 *	 materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *	 contributors may be used to endorse or promote products derived from this
 *	 software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *	 Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *	 engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 * @defgroup nrf_twi_master_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Example Application main file.
 *
 * This file contains the source code for a sample application using TWI.
 *
 * @image html example_board_setup_a.jpg "Use board setup A for this example."
 */


// #define AdjustOffsetGain // Kollas!!!!

//#define DebugEEPROM		// EEPROM Readout
//#define DebugVDDComp	// VddCompGrad & VddCompOffs
//#define DebugSensordata // Sensor & calculated offsets


#include <stdio.h>
#include "boards.h"
#include "app_util_platform.h"
#include "app_error.h"
#include "nrf_delay.h"
#include "HTPA32x32.h"
#include "app_twi.h"
#include "globals.h"
#include "HTPA32x32_Table.c"
#include "m_ApplicationInterface.h"


#define NRF_LOG_MODULE_NAME "HTPA32x32"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"


char Resolution;
unsigned short PTATUpperBlock[4];			//PTAT block 
unsigned short PTATLowerBlock[4];

// 32x32 matrices
// **************
unsigned short SensorVij[NUMBER_OF_ROWS][PIX_PER_ROW]; // 2x32x32 = 2k
unsigned short ElOffMatrixVij[NUMBER_OF_ROWS][PIX_PER_ROW]; // 2x32x32 = 2k
signed short ThGrad[NUMBER_OF_ROWS][PIX_PER_ROW]; // 2x32x32 = 2k
signed short ThOff[NUMBER_OF_ROWS][PIX_PER_ROW]; // 2x32x32 = 2k
unsigned long PixC[NUMBER_OF_ROWS][PIX_PER_ROW];	// 4x32x32 = 4k
signed short VddCompOff[NUMBER_OF_ROWS][PIX_PER_ROW]; // 2x32x32 = 2k
signed short VddCompGrad[NUMBER_OF_ROWS][PIX_PER_ROW]; // 2x32x32 = 2k
// ************** TOTAL: 18k

// Temporra debugvariabler fr bearbetning i excel
// ************************************************
signed short TEMP_ThermalGradCompTable[NUMBER_OF_ROWS][PIX_PER_ROW];	// 2x32x32 = 2k
signed short TEMP_ThermalOffsCompTable[NUMBER_OF_ROWS][PIX_PER_ROW];	// 2x32x32 = 2k
signed short TEMP_VddCompTable[NUMBER_OF_ROWS][PIX_PER_ROW];	// 2x32x32 = 2k
signed short TEMP_IndataToLookupTable[NUMBER_OF_ROWS][PIX_PER_ROW];	// 2x32x32 = 2k
// ************************	TOTAL: 16k

float PixCMin;
float	PixCMax;
char gradScale;
unsigned char TableNumberSensor;
unsigned char epsilon;
uint8_t SetMBITCalib;
uint8_t SetBIASCalib;
uint8_t SetCLKCalib;
uint8_t SetBPACalib;
uint8_t SetPUCalib;
float PTATGrad;
float PTATOff;
unsigned char MBITUser;
unsigned char BIASUser;
unsigned char CLKUser;
unsigned char BPAUser;
unsigned char PUUser;
unsigned char NrOfDefPix;
unsigned long DeviceID;
signed char GlobalOffset;
unsigned short GlobalGain;
// unsigned short eloffstack[ELAMOUNT][StackSize];	//ELAMOUNT=256; StackSize=16 (arbitrary, should be chosen to match the data memory of the controller)
unsigned char VddScalingGrad;
unsigned char VddScalingOff;
unsigned short VddRef;

float PTATGrad;
float PTATOff;
unsigned short Ta;		// Ambient temperature
unsigned long PTATLong;

// variabler fr att hantera defekta pixlar
unsigned short addressOfDefPix[52];
unsigned char	deadPixMask[52];

#define pixUpperLeft -33
#define pixUpper -32
#define pixUpperRight -31
#define pixLeft -1
#define pixRight 1
#define pixLowerLeft 31
#define pixLower 32
#define pixLowerRight 33

typedef struct
{
	unsigned short addressOfDefPix;
	unsigned char deadMaskPixUpperRight;
	unsigned char deadMaskPixUpper;
	unsigned char deadMaskPixUpperLeft;
	unsigned char deadMaskPixRight;
	unsigned char deadMaskPixLeft;
	unsigned char deadMaskPixLowerRight;
	unsigned char deadMaskPixLower;
	unsigned char deadMaskPixLowerLeft;
	unsigned char numberOfMasks;
} deadPixStruct;

deadPixStruct deadPix[52];
/*
unsigned char THdeadMaskPixUpperRight;
unsigned char THdeadMaskPixUpper;
unsigned char THdeadMaskPixUpperLeft;
unsigned char THdeadMaskPixRight;
unsigned char THdeadMaskPixLeft;
unsigned char THdeadMaskPixLowerRight;
unsigned char THdeadMaskPixLower;
unsigned char THdeadMaskPixLowerLeft;

unsigned char BHdeadMaskPixUpperRight;
unsigned char BHdeadMaskPixUpper;
unsigned char BHdeadMaskPixUpperLeft;
unsigned char BHdeadMaskPixRight;
unsigned char BHdeadMaskPixLeft;
unsigned char BHdeadMaskPixLowerRight;
unsigned char BHdeadMaskPixLower;
unsigned char BHdeadMaskPixLowerLeft;
*/


// Buffer for data read from sensors.
uint8_t rxBuffer[512];	// 256 x 16bit VDDcomp is the largest read operation




/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void HTPA32x32_SENSOR_writeConfig(uint8_t regAddress, uint8_t command)
{
	static uint8_t data[2];

	data[0] = regAddress;
	data[1] = command;
		
//	printf("Writing SensorConfig [0x%02X] = 0x%02X\n", regAddress, command);
//	nrf_delay_ms(15);
	
	app_twi_transfer_t HTPA32x32_init_device_1[1] = { APP_TWI_WRITE(HTPA32x32_SENSOR_ADDR, data, sizeof(data), 0) };
	APP_ERROR_CHECK(app_twi_perform(&m_app_twi, HTPA32x32_init_device_1, 1, NULL));
	
	nrf_delay_ms(5); // All configuation writes must have a 5ms pause
}


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void HTPA32x32_SENSOR_readData(unsigned char address,unsigned char *rxBuffer, unsigned short numbytes)
{
	static uint8_t data[1];
	
	data[0] = address;

	app_twi_transfer_t HTPA32x32_read_device12[2] = { HTPA32x32_SENSOR_READ(data, rxBuffer, numbytes) };
	APP_ERROR_CHECK(app_twi_perform(&m_app_twi, HTPA32x32_read_device12, 2, NULL));
}


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void HTPA32x32_MEMORY_writeData(unsigned short address,unsigned char *txBuffer, unsigned short numbytes)
{
	static uint8_t data[2];
	unsigned char HighAdd,LowAdd;
	
	LowAdd = (unsigned char) (address&0xFF);
	HighAdd = (unsigned char) ((address&0xFF00)>>8);
	
	data[0]=HighAdd;
	data[1]=LowAdd;
	
	for(int i=0;i<numbytes;i++)
		data[i+2]=txBuffer[i];
	
	app_twi_transfer_t HTPA32x32_write_device[1] = { APP_TWI_WRITE(HTPA32x32_MEMORY_ADDR, data, numbytes + 2 , 0) };
	APP_ERROR_CHECK(app_twi_perform(&m_app_twi, HTPA32x32_write_device, 1, NULL));
		
	nrf_delay_ms(5);		//give the sensor some time to write to the EEPROM				// Kolla tiden, behvs denna eller r den fr liten???
}


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void HTPA32x32_MEMORY_readData(unsigned short address,unsigned char *rxBuffer, unsigned short numbytes)
{
	uint8_t data[2];
	unsigned char HighAdd,LowAdd;
	
	LowAdd = (unsigned char) (address&0xFF);
	HighAdd = (unsigned char) ((address&0xFF00)>>8);
	
	data[0]=HighAdd;
	data[1]=LowAdd;

	app_twi_transfer_t HTPA32x32_read_device[2] = { HTPA32x32_MEMORY_READ(data, rxBuffer, numbytes) };
	APP_ERROR_CHECK(app_twi_perform(&m_app_twi, HTPA32x32_read_device, 2, NULL));
}


/********************************************************************
 * Function:		  void InitBPATRIM(unsigned char user)
 *
 * Description:	  
 *
 * Dependencies:	 user:	0-> Setting during Calibration
 *							1-> Setting from user
 *******************************************************************/
void InitBPATRIM(unsigned char user)
{
	char tempData;

	if(user)
		tempData = BPAUser;
	else	
		tempData = SetBPACalib;
	
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_TRIM_5, tempData);
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_TRIM_6, tempData);

}


/********************************************************************
 * Function:		  void InitPUTRIM(unsigned char user)
 *
 * Description:	  
 *
 * Dependencies:	 user:	0-> Setting during Calibration
 *							1-> Setting from user
 *******************************************************************/
void InitPUTRIM(unsigned char user)
{
	uint8_t tempData;
	
	if(user)
		tempData = PUUser;
	else	
		tempData = SetPUCalib;
	
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_TRIM_7, tempData);

}


/********************************************************************
 * Function:		  void InitPUTRIM(unsigned char user)
 *
 * Description:	  
 *
 * Dependencies:	 user:	0-> Setting during Calibration
 *							1-> Setting from user
 *******************************************************************/
void InitCLKRIM(unsigned char user)
{
		uint8_t tempData;
	
	if(user)
		tempData = CLKUser;
	else	
		tempData = SetCLKCalib;
	
	
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_TRIM_4, tempData); // Kollas: var RESOLUTION?!?!? = 0x30 skrevs!
}


/********************************************************************
 * Function:		  void InitBIASTRIM(unsigned char user)
 *
 * Description:	  
 *
 * Dependencies:	 user:	0-> Setting during Calibration
 *							1-> Setting from user
 *******************************************************************/
void InitBIASTRIM(unsigned char user)
{
	char tempData;
	
	if(user)
		tempData = BIASUser;
	else	
		tempData = SetBIASCalib;

	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_TRIM_2, tempData);
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_TRIM_3, tempData);
}


/********************************************************************
 * Function:		  void InitMBITTRIM(unsigned char user)
 *
 * Description:	  
 *
 * Dependencies:	 user:	0-> Setting during Calibration
 *							1-> Setting from user
 *******************************************************************/
void InitMBITTRIM(unsigned char user)
{
	char tempData;

	if(user)
	{
		tempData = MBITUser;
		Resolution = MBITUser+4;
	}
	else
	{	
		tempData = SetMBITCalib;
		Resolution = SetMBITCalib+4;				// Sannolikt skumt! Resolution anvnds dock bara p ett nnu skummare stt... som r borttaget
	}
	
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_TRIM_1, tempData);
}


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
uint32_t HTPA32x32_read_DeviceID()
{
	// Read DeviceID
	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_DevID,rxBuffer, sizeof(long));		// 4 bytes
	return (uint32_t) (rxBuffer[0] + 256*rxBuffer[1] + 256*256*rxBuffer[2] + 256*256*256*rxBuffer[3]);			
}

/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void printRawEEPROMData(void)
{
	unsigned short address;
	unsigned char data;
	
	uint16_t x,y;
	
	printf("\r\n-- Sensor EEPROM RAW dump start --\r\n");

	address = 0;
	x = 0;
	
	for (y = 0; y < 1024; y++)
	{
		address = y *16;
		printf("%04xh: ",address);
		
		for (x = 0; x < 16; x++)
		{
			address = y*16 + x;
			
			HTPA32x32_MEMORY_readData(address,rxBuffer, sizeof(char));		// 1 byte
			data = rxBuffer[0];
			
			printf("%02x ",data);

//	  	printf("%04x ",address);
			
			nrf_delay_ms(1);
		}
		
		x = 0;
		
		printf("\r\n");
	}
	
	
		printf("\r\n-- Sensor EEPROM RAW dump end --\r\n");
	
}

#ifdef DebugEEPROM
/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void printEEPROMCalibrationData(void)
{
	unsigned char i,j;
	
	printf("\r\n-- EEPROM Calibration data readout start --\r\n");
	
	printf("[0x%04X] PixCMin: %f\n",HTPA32x32_REGM_PixCMin, PixCMin);
	nrf_delay_ms(10);
	printf("[0x%04X] PixCMax: %f\n",HTPA32x32_REGM_PixCMax, PixCMax);
	nrf_delay_ms(10);
	printf("[0x%04X] gradScale: %d (0x%02X)\n", HTPA32x32_REGM_GradScale, gradScale, gradScale);
	nrf_delay_ms(10);
	printf("[0x%04X] TableNumberSensor: %d (0x%02X)\n",HTPA32x32_REGM_TableNumber, TableNumberSensor, TableNumberSensor);
	nrf_delay_ms(10);
	printf("[0x%04X] epsilon: %d (0x%02X)\n", HTPA32x32_REGM_Epsilon, epsilon, epsilon);
	nrf_delay_ms(10);
	printf("[0x%04X] SetMBITCalib: %d (0x%02X)\n",HTPA32x32_REGM_MBITCal, SetMBITCalib, SetMBITCalib);
	nrf_delay_ms(10);
	printf("[0x%04X] SetBIASCalib: %d (0x%02X)\n",HTPA32x32_REGM_BIASCal, SetBIASCalib, SetBIASCalib);
	nrf_delay_ms(10);
	printf("[0x%04X] SetCLKCalib: %d (0x%02X)\n",HTPA32x32_REGM_CLKCal, SetCLKCalib, SetCLKCalib);
	nrf_delay_ms(10);
	printf("[0x%04X] SetBPACalib: %d (0x%02X)\n",HTPA32x32_REGM_BPACal, SetBPACalib, SetBPACalib);
	nrf_delay_ms(10);
	printf("[0x%04X] SetPUCalib: %d (0x%02X)\n",HTPA32x32_REGM_PUCal, SetPUCalib, SetPUCalib);
	nrf_delay_ms(10);
	nrf_delay_ms(10);
	printf("[0x%04X] PTATGrad: %f\n",HTPA32x32_REGM_PTATGrad, PTATGrad);
	nrf_delay_ms(10);
	printf("[0x%04X] PTATOff: %f\n",HTPA32x32_REGM_PTATOffs, PTATOff);
	nrf_delay_ms(10);
	
	printf("[0x%04X] VddScalingGrad: %u (0x%02X)\n",HTPA32x32_REGM_VddScalingGrad, VddScalingGrad, VddScalingGrad);
	nrf_delay_ms(10);
	printf("[0x%04X] VddScalingOff: %u (0x%02X)\n",HTPA32x32_REGM_VddScalingOff, VddScalingOff, VddScalingOff);
	nrf_delay_ms(10);
	printf("[0x%04X] GlobalOffset: %d (0x%02X)\n",HTPA32x32_REGM_GlobalOffset, GlobalOffset, GlobalOffset);
	nrf_delay_ms(10);
	printf("[0x%04X] GlobalGain: %u (0x%04X)\n",HTPA32x32_REGM_GlobalGain, GlobalGain, GlobalGain);
	nrf_delay_ms(10);

	printf("[0x%04X] MBITUser: %d (0x%02X)\n",HTPA32x32_REGM_MBITUser, MBITUser, MBITUser);
	nrf_delay_ms(10);
	printf("[0x%04X] BIASUser: %d (0x%02X)\n",HTPA32x32_REGM_BIASUser, BIASUser, BIASUser);
	nrf_delay_ms(10);
	printf("[0x%04X] CLKUser: %d (0x%02X)\n",HTPA32x32_REGM_CLKUser, CLKUser, CLKUser);
	nrf_delay_ms(10);
	printf("[0x%04X] BPAUser: %d (0x%02X)\n",HTPA32x32_REGM_BPAUser, BPAUser, BPAUser);
	nrf_delay_ms(10);
	printf("[0x%04X] PUUser: %d (0x%02X)\n",HTPA32x32_REGM_PUUser, PUUser, PUUser);
	nrf_delay_ms(10);

	printf("[0x%04X] DeviceID: %lu (0x%08lx)\n",HTPA32x32_REGM_DevID, DeviceID, DeviceID);
	nrf_delay_ms(10);
	printf("[0x%04X] NrOfDefPix: %d (0x%02X)\n",HTPA32x32_REGM_NrOfDefPix, NrOfDefPix, NrOfDefPix);
	nrf_delay_ms(10);

	printf("ThGrad (16-bit signed) raw data from EEPROM :\r\n");

	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%6d ",ThGrad[i][j]);
			// printf("%6d ",j + 32*i);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}

	printf("ThOff (16-bit signed) raw data from EEPROM :\r\n");

	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%6d ",ThOff[i][j]);
			//printf("%6d ",j + 32*i);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}

	printf("PixC (16-bit unsigned raw data in EEPROM -> Unsigned long as of calculation in datasheet chapter 8.5) :\r\n");
	
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%9lu ",PixC[i][j]);
			//printf("%6lu ",j + 32*i);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}
	
	
	printf("-- EEPROM Calibration data readout end --\r\n");

}
#endif

#ifdef DebugSensordata
/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void printSensorData(void)
{
	unsigned char x,y;
	unsigned char i,j;
	uint16_t pixelposition;
	
	printf("\r\n-- Sensor data start --\r\n");
	
	printf("PTATUpperBlock:\r\n");
	for (x = 0; x < 4; x++)
	{
		printf("%5u ",PTATUpperBlock[x]);
		nrf_delay_ms(1);
	}
	printf("\r\n");
	
	printf("PTATLowerBlock:\r\n");	
	for (x = 0; x < 4; x++)
	{
		printf("%5u ",PTATLowerBlock[x]);
		nrf_delay_ms(1);
	}
	printf("\r\n");

	printf("=====> PTATlong(avg) = %lu Ambient Temperature: %5u dK (%.2f deg. C)\r\n",PTATLong, Ta,(Ta-2730.0)/10.0);	
	printf("\r\n");

	printf("Sensor raw values:\r\n");
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%4u ",SensorVij[i][j]);
			nrf_delay_ms(5);
		}
		printf("\r\n");
	}
	
	printf("Electrical Offsets used (ElOffMatrixVij):\r\n");
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%4d ",ElOffMatrixVij[i][j]);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}

	printf("Thermal Gradients compensation used (TEMP_ThermalGradCompTable):\r\n");
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%4d ",TEMP_ThermalGradCompTable[i][j]);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}

	printf("Thermal Offsets compensation used (TEMP_ThermalOffsCompTable):\r\n");
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%4d ",TEMP_ThermalOffsCompTable[i][j]);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}
	
	printf("VDD Offsets compensation used (TEMP_VddCompTable):\r\n");
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%4d ",TEMP_VddCompTable[i][j]);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}
	
	printf("Indata to LUT used (TEMP_IndataToLookupTable):\r\n");
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%4d ",TEMP_IndataToLookupTable[i][j]);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}
		
	printf("SensorImage final temperatures in deg. C:\r\n");
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{
			printf("%4d ",SensorImage[i][j]);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}
	
	printf("-- Sensor data end --\r\n");
}
#endif

/********************************************************************
 * Function:		  void readCalibrationData(void)
 *
 * Description:	  read all required calibration data
 *
 * Dependencies:	 
 *******************************************************************/
void readEEPROMCalibrationData(void)
{
		static float tempFloat[2];
//		static float c,d,e,f;
//		static unsigned long a,b;
//		int pixelposition;
		int i, j, readdataposition, pixelposition_i;
		uint16_t PixCRaw;
		uint16_t raw_value;
		uint16_t bufferposition;
	
		float slope;
		float epsilonfactor;
		float globalgainfactor;
	
	
		//Read AddrPixCMin
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_PixCMin,rxBuffer, sizeof(float)*2);	// 4 bytes HTPA32x32_REGM_PixCMax follows // ?!?!? Kollas Sizeof(float)
		memcpy(&tempFloat,rxBuffer, sizeof(float)*2);
		PixCMin = tempFloat[0];
		PixCMax = tempFloat[1];
	
		//Read AdrGradScale
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_GradScale,rxBuffer, sizeof(char));		// 1 byte
		gradScale = rxBuffer[0];
	
		//Read AdrTN
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_TableNumber,rxBuffer, sizeof(char)*2);	// 2 bytes
		TableNumberSensor = rxBuffer[0] + 256*rxBuffer[1];
		
		//Read AdeEpsilon
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_Epsilon,rxBuffer, sizeof(char));	// 1 byte
		epsilon = rxBuffer[0];
	
		//Read AdrMBITPixC and on
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_MBITCal,rxBuffer, sizeof(char)*5); // 5 bytes in series (HTPA32x32_REGM_MBITPixC, HTPA32x32_REGM_BIASPixC, HTPA32x32_REGM_CLKPixC, HTPA32x32_REGM_BPAPixC and HTPA32x32_REGM_PUPixC)
		SetMBITCalib = rxBuffer[0];
		SetBIASCalib = rxBuffer[1];
		SetCLKCalib = rxBuffer[2];
		SetBPACalib = rxBuffer[3];
		SetPUCalib = rxBuffer[4];
		
		//Read AdPTATGrad
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_PTATGrad,rxBuffer, sizeof(float)*2);	// 4 bytes, HTPA32x32_REGM_PTATOffs follows (4 bytes more)
		memcpy(&tempFloat,rxBuffer, sizeof(float)*2);
		PTATGrad = tempFloat[0];
		PTATOff = tempFloat[1];	 

		// Read DeviceID
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_DevID,rxBuffer, sizeof(long));		// 4 bytes
		DeviceID = rxBuffer[0] + 256*rxBuffer[1] + 256*256*rxBuffer[2] + 256*256*256*rxBuffer[3];		

		//Read AdrMBITUser
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_MBITUser,rxBuffer, sizeof(char));
		MBITUser = rxBuffer[0];
		if((MBITUser < 0x09)||(MBITUser > 0x0c)) //EEPROM empty or setting out of spec -> set to default
		{	
			MBITUser = (unsigned char) HTPA32x32_MBITTRIMDefault;
			HTPA32x32_MEMORY_writeData(HTPA32x32_REGM_MBITUser, &MBITUser, sizeof(char));
		}
		
		//Read HTPA32x32_REGM_BIASUser
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_BIASUser,rxBuffer, sizeof(char));
		BIASUser = rxBuffer[0];
		if((!BIASUser)||(BIASUser > 0x1F))
		{	
			BIASUser = (unsigned char) HTPA32x32_BIAScurrentDefault;
			HTPA32x32_MEMORY_writeData(HTPA32x32_REGM_BIASUser, &BIASUser, sizeof(char));
		}
		
		//Read AdrCLKUser
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_CLKUser,rxBuffer, sizeof(char));
		CLKUser = rxBuffer[0];
		if(CLKUser > 0x3F)
		{	
			CLKUser = (unsigned char) HTPA32x32_CLKTRIMDefault;
			HTPA32x32_MEMORY_writeData(HTPA32x32_REGM_CLKUser, &CLKUser, sizeof(char));
		}
		
		//Read AdrBPAUser
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_BPAUser,rxBuffer, sizeof(char));
		BPAUser = rxBuffer[0];
		if((!BPAUser)||(BPAUser > 0x1F))
		{	
			BPAUser = (unsigned char) HTPA32x32_BPATRIMDefault;
			HTPA32x32_MEMORY_writeData(HTPA32x32_REGM_BPAUser, &BPAUser, sizeof(char));
		}
		
		//Read AdrPUUser
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_PUUser,rxBuffer, sizeof(char));
		PUUser = rxBuffer[0];
		if((!PUUser)||(PUUser > 0x1F))
		{	
			PUUser = (unsigned char) HTPA32x32_PUTRIMDefault;
			HTPA32x32_MEMORY_writeData(HTPA32x32_REGM_PUUser, &PUUser, sizeof(char));
		}
		
		// Set default data if AdjustOffsetGain is not used
		GlobalOffset=0;
		GlobalGain = 10000;
		
#ifdef AdjustOffsetGain
		
		// Read GlobalOffset
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_GlobalOffset,rxBuffer, sizeof(char));
		GlobalOffset = rxBuffer[0];
		if(GlobalOffset==(char)0xFF) // If values are not set before, do it now!
		{
			GlobalOffset=0;
			HTPA32x32_MEMORY_writeData(HTPA32x32_REGM_GlobalOffset, (unsigned char *) &GlobalOffset, sizeof(char));
		}

		// Read GlobalGain
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_GlobalGain,rxBuffer, sizeof(char)*2);
		GlobalGain = rxBuffer[0] + 256*rxBuffer[1];
		if((GlobalGain==0x0000)||(GlobalGain==0xFFFF)) 	// If values are not set before, do it now!
		{
			GlobalGain = 10000;
			HTPA32x32_MEMORY_writeData(HTPA32x32_REGM_GlobalGain, (unsigned char *) &GlobalGain, sizeof(char)*2);
		}
#endif	
		
		
		//now load the Thermal-Grad and -Offset, Observe data order (both arrays) in datasheet chapter 8.0. Word order 0..511, 1023..512 is unclear! Indications suggest order is reversed in middle of EEPROM data!
		// Total 2032 bytes = 1024 x 16-bit signed values per matrix
		
	for(i = 0; i < NUMBER_OF_ROWS; i++)
	{
		for(j = 0; j < PIX_PER_ROW; j++)
		{
			readdataposition = HTPA32x32_REGM_Th1 + (i * PIX_PER_ROW + j) * 2;
			
//			printf("Reading ThGrad[%u][%u] (Memory address: 0x%04X) - ", i, j, readdataposition);
//			nrf_delay_ms(15);
			
			HTPA32x32_MEMORY_readData(readdataposition,rxBuffer,sizeof(char)*2);

			if(i < 16)	// Top half
			{
				
//				pixelposition = i * PIX_PER_ROW + j;
//				printf("Writing ThGrad[%u][%u] (Pixel %u)\n",i,j,pixelposition);
//				nrf_delay_ms(15);
				
				ThGrad[i][j]= rxBuffer[0] + 256*rxBuffer[1];
			}
			else	// Bot half
			{
				pixelposition_i = 47 - i;
				
//				pixelposition = pixelposition_i * PIX_PER_ROW + j;
//				printf("Writing ThGrad[%u][%u] (Pixel %u)\n",pixelposition_i,j,pixelposition);
//				nrf_delay_ms(15);
				
				ThGrad[pixelposition_i][j]= rxBuffer[0] + 256*rxBuffer[1];
			}
			
			
			readdataposition = HTPA32x32_REGM_Th2 + (i * PIX_PER_ROW + j) * 2;
			
//			printf("Reading ThOff[%u][%u] (Memory address: 0x%04X) - ", i, j, readdataposition);
//			nrf_delay_ms(15);
			
			HTPA32x32_MEMORY_readData(readdataposition,rxBuffer,sizeof(char)*2);
			
			if(i < 16)	// Top half
			{
				
//				pixelposition = i * PIX_PER_ROW + j;
//				printf("Writing ThOff[%u][%u] (Pixel %u)\n",i,j,pixelposition);
//				nrf_delay_ms(15);
				
				ThOff[i][j]= rxBuffer[0] + 256*rxBuffer[1];
			}
			else	// Bot half
			{
				pixelposition_i = 47 - i;
				
//				pixelposition = pixelposition_i * PIX_PER_ROW + j;
//				printf("Writing ThOff[%u][%u] (Pixel %u)\n",pixelposition_i,j,pixelposition);
//				nrf_delay_ms(15);
				
				ThOff[pixelposition_i][j]= rxBuffer[0] + 256*rxBuffer[1];
			}
		}
	}
		//and finally the PixC.... *PROBABLY* the same data order as above.... not specified in the datasheet...
		
// PixCij (= PixC array) is the sensitivity coefficient for each pixel
// Pij is the stored sensitivity coefficient scaled to 16 bit
// PixCmin is the minimum sensitivity coefficient, used for scaling
// PixCmax is the maximum sensitivity coefficient, used for scaling
// epsilon is the emissivity factor
// GlobalGain is a factor for fine tuning of the sensitivity for all Pixel

		slope = (PixCMax - PixCMin)/((double) 65535.0);
		epsilonfactor = epsilon / 100.0;
				
#ifdef AdjustOffsetGain
		//globalgainfactor = GlobalGain / 10000.0;
#else
		globalgainfactor = 1;
#endif

//	printf("Calculating PixC using epsilonfactor %f, GlobalGainFactor: %f\n",epsilonfactor, globalgainfactor);
				
	for(i = 0; i < NUMBER_OF_ROWS; i++)
	{
		for(j = 0; j < PIX_PER_ROW; j++)
		{
			readdataposition = HTPA32x32_REGM_PixC + (i * PIX_PER_ROW + j) * 2;
			
//			printf("Reading PixC[%u][%u] (Memory address: 0x%04X) - ", i, j, readdataposition);
//			nrf_delay_ms(15);
			
			HTPA32x32_MEMORY_readData(readdataposition,rxBuffer,sizeof(char)*2);

			if(i < 16)	// Top half
			{			
//				pixelposition = i * PIX_PER_ROW + j;
//				printf("Writing PixC[%u][%u] (Pixel %u)\n",i,j,pixelposition);
//				nrf_delay_ms(15);
				
				PixCRaw = rxBuffer[0] + 256*rxBuffer[1];
				
				PixC[i][j] = (unsigned long) ((PixCRaw * slope + PixCMin) * epsilonfactor * globalgainfactor + (float)0.5);
			}
			else	// Bot half
			{				
				pixelposition_i = 47 - i;
				
//				pixelposition = pixelposition_i * PIX_PER_ROW + j;	
//				printf("Writing PixC[%u][%u] (Pixel %u)\n",pixelposition_i,j,pixelposition);
//				nrf_delay_ms(15);
				
				PixCRaw = rxBuffer[0] + 256*rxBuffer[1];
				
				PixC[pixelposition_i][j] = (unsigned long) ((PixCRaw * slope + PixCMin) * epsilonfactor * globalgainfactor + (float)0.5);
			}
		}
	}

	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_VddScalingGrad,rxBuffer, sizeof(char));
	VddScalingGrad = rxBuffer[0];		
#ifdef DebugVDDComp
	printf("VddScalingGrad: %u (0x%02x)\n", VddScalingGrad, VddScalingGrad);	
#endif
	
//	for(i=0;i<(unsigned short)ELAMOUNT;i++)	// 256 normally
//	{
//		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_VddCompGrad+i*2,rxBuffer, sizeof(char)*2);
////		VddCompGrad[i][j] = rxBuffer[0] + 256*rxBuffer[1];
//	}
	
	// 8 x 32 = 256 values, 2 bytes/value => 512 bytes to read, signed 16-bit data (0x0340 -> 0x053F in EEPROM)
	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_VddCompGrad,rxBuffer, 512);
	
#ifdef DebugVDDComp
	printf("VddCompGrad raw data from EEPROM:\r\n");	
#endif
	
	bufferposition = 0;
	for (i = 0; i < 8; i++)
	{
		for (j = 0; j < 32; j++)
		{			
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];
			
#ifdef DebugVDDComp			
			printf("%6d ",raw_value);
			nrf_delay_ms(1);
#endif
			
			bufferposition += 2;
		}
#ifdef DebugVDDComp		
		printf("\r\n");
#endif
	}
	
	bufferposition = 0;
	
	for(i = 0; i < 4; i++)	// First 4 rows are in TOP -> BOT order 
	{
		for(j = 0; j < PIX_PER_ROW; j++) 
		{
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading VddCompGrad pos: %u ([%u][%u]), writing VddCompGrad%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,i,j, i,i+4,i+8,i+12,j,i*32 + j, (i+4)*32 + j, (i+8)*32 + j, (i+12)*32 + j);
//			nrf_delay_ms(15);
			
			VddCompGrad[i][j]= raw_value;	// Block 0 
			VddCompGrad[i + 4][j]= raw_value; // Block 1
			VddCompGrad[i + 8][j]= raw_value; // Block 2
			VddCompGrad[i + 12][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}

	// Keep bufferposition and continue in reverse order
	
	for(i = 0; i < 4; i++)	// Second 4 rows are in BOT -> TOP order 
	{
		for(j = 0; j < PIX_PER_ROW; j++) 
		{
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading VddCompGrad pos: %u ([%u][%u]), writing VddCompGrad[%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,i+4,j,31 - i,27 - i,23 - i,19 - i,j ,(31-i)*32 + j, (27-i)*32 + j, (23-i)*32 + j, (19-i)*32 + j);
//			nrf_delay_ms(15);
			
			VddCompGrad[31 - i][j]= raw_value;	// Block 0
			VddCompGrad[27 - i][j]= raw_value; // Block 1
			VddCompGrad[23 - i][j]= raw_value; // Block 2
			VddCompGrad[19 - i][j]= raw_value; // Block 3
			 
			bufferposition += 2;
		}
	}

#ifdef DebugVDDComp	
	printf("VddCompGrad final matrix:\r\n");	
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{			
			printf("%6d ",VddCompGrad[i][j]);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}	
#endif
	
	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_VddScalingOff,rxBuffer, sizeof(char));
	VddScalingOff = rxBuffer[0];		
#ifdef DebugVDDComp	
	printf("VddScalingOff: %u (0x%02x)\n", VddScalingOff, VddScalingOff);	
#endif
	
	// 8 x 32 = 256 values, 2 bytes/value => 512 bytes to read, signed 16-bit data (0x0540 -> 0x073F in EEPROM)
	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_VddCompOff,rxBuffer, 512);
	
#ifdef DebugVDDComp	
	printf("VddCompOff raw data from EEPROM:\r\n");	
#endif
	
	bufferposition = 0;
	for (i = 0; i < 8; i++)
	{
		for (j = 0; j < 32; j++)
		{			
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];

#ifdef DebugVDDComp			
			printf("%6d ",raw_value);
			nrf_delay_ms(1);
#endif
			
			bufferposition += 2;
		}
#ifdef DebugVDDComp		
		printf("\r\n");
#endif
	}
	
	bufferposition = 0;
	
	for(i = 0; i < 4; i++)	// First 4 rows are in TOP -> BOT order 
	{
		for(j = 0; j < PIX_PER_ROW; j++) 
		{
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading VddCompOff pos: %u ([%u][%u]), writing VddCompOff[%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,i,j, i,i+4,i+8,i+12,j,i*32 + j, (i+4)*32 + j, (i+8)*32 + j, (i+12)*32 + j);
//			nrf_delay_ms(15);
			
			VddCompOff[i][j]= raw_value;	// Block 0 
			VddCompOff[i + 4][j]= raw_value; // Block 1
			VddCompOff[i + 8][j]= raw_value; // Block 2
			VddCompOff[i + 12][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}

	// Keep bufferposition and continue in reverse order
	
	for(i = 0; i < 4; i++)	// Second 4 rows are in BOT -> TOP order 
	{
		for(j = 0; j < PIX_PER_ROW; j++) 
		{
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading VddCompOff pos: %u ([%u][%u]), writing VddCompOff[%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,i+4,j,31 - i,27 - i,23 - i,19 - i,j ,(31-i)*32 + j, (27-i)*32 + j, (23-i)*32 + j, (19-i)*32 + j);
//			nrf_delay_ms(15);
			
			VddCompOff[31 - i][j]= raw_value;	// Block 0
			VddCompOff[27 - i][j]= raw_value; // Block 1
			VddCompOff[23 - i][j]= raw_value; // Block 2
			VddCompOff[19 - i][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}

#ifdef DebugVDDComp	
	printf("VddCompOff final matrix:\r\n");	
	for (i = 0; i < 32; i++)
	{
		for (j = 0; j < 32; j++)
		{			
			printf("%6d ",VddCompOff[i][j]);
			nrf_delay_ms(1);
		}
		printf("\r\n");
	}
#endif
	
	// if(SENSORREV1)	
	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_VddCalib,rxBuffer, sizeof(char)*2);
	memcpy((char*)&VddRef,rxBuffer,sizeof(short));
		
	
	// Read number of defect pixels
	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_NrOfDefPix,rxBuffer, sizeof(char));		// 1 byte
	NrOfDefPix = rxBuffer[0];	
	if(NrOfDefPix!=0)
	{
		
		//read data
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_DefPixAddr,rxBuffer, sizeof(short)*NrOfDefPix);
		memcpy((char*)&addressOfDefPix,rxBuffer, sizeof(short)*NrOfDefPix);
		
		HTPA32x32_MEMORY_readData(HTPA32x32_REGM_DefPixMask,rxBuffer, sizeof(char)*NrOfDefPix);
		memcpy((char*)&deadPixMask,rxBuffer, sizeof(char)*NrOfDefPix);
		
		for(i=0;i<NrOfDefPix;i++)
		{
			deadPix[i].addressOfDefPix = addressOfDefPix[i];
			if(addressOfDefPix[i]<512)				//Upper half
			{
				deadPix[i].deadMaskPixUpperRight = (deadPixMask[i]&0x02)>>1;
				deadPix[i].deadMaskPixUpper = (deadPixMask[i]&0x01);
				deadPix[i].deadMaskPixUpperLeft = (deadPixMask[i]&0x80)>>7;
				deadPix[i].deadMaskPixRight = (deadPixMask[i]&0x04)>>2;
				deadPix[i].deadMaskPixLeft = (deadPixMask[i]&0x60)>>6;
				deadPix[i].deadMaskPixLowerRight = (deadPixMask[i]&0x08)>>3;
				deadPix[i].deadMaskPixLower = (deadPixMask[i]&0x10)>>4;
				deadPix[i].deadMaskPixLowerLeft = (deadPixMask[i]&0x20)>>5;
				deadPix[i].numberOfMasks = deadPix[i].deadMaskPixUpperRight+deadPix[i].deadMaskPixUpper+deadPix[i].deadMaskPixUpperLeft+deadPix[i].deadMaskPixRight+deadPix[i].deadMaskPixLeft+deadPix[i].deadMaskPixLowerRight+deadPix[i].deadMaskPixLower+deadPix[i].deadMaskPixLowerLeft;
				
			}
			else
			{
				deadPix[i].deadMaskPixUpperRight = (deadPixMask[i]&0x08)>>3;
				deadPix[i].deadMaskPixUpper = (deadPixMask[i]&0x10)>>4;
				deadPix[i].deadMaskPixUpperLeft = (deadPixMask[i]&0x20)>>5;
				deadPix[i].deadMaskPixRight = (deadPixMask[i]&0x04)>>2;
				deadPix[i].deadMaskPixLeft = (deadPixMask[i]&0x40)>>6;
				deadPix[i].deadMaskPixLowerRight = (deadPixMask[i]&0x02)>>1;
				deadPix[i].deadMaskPixLower = (deadPixMask[i]&0x01);
				deadPix[i].deadMaskPixLowerLeft = (deadPixMask[i]&0x80)>>7;
				deadPix[i].numberOfMasks = deadPix[i].deadMaskPixUpperRight+deadPix[i].deadMaskPixUpper+deadPix[i].deadMaskPixUpperLeft+deadPix[i].deadMaskPixRight+deadPix[i].deadMaskPixLeft+deadPix[i].deadMaskPixLowerRight+deadPix[i].deadMaskPixLower+deadPix[i].deadMaskPixLowerLeft;
				
			}
		}
		
	} 
}


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void startSensorForElectricalOffsetReadout(void)
{	
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_CONFIGURATION, 0x0B); //  [2xRFU][2xBlock][1 0 1 1] = Block is ignored, Start + not VDD + BLIND + WAKEUP => PTAT is sampled
}


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void waitForSensorReady(void)
{
	unsigned char result = 0;
	
	do
	{
		HTPA32x32_SENSOR_readData(HTPA32x32_REGS_STATUS, &result, 1) ;
	} while(!(result&0x01));	
	
}


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void startSensor(uint8_t blockNr)
{
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_CONFIGURATION, (0x09 | (blockNr << 4)));		// [2xRFU][2xBlock][1 0 0 1] = Start + not VDD + not BLIND + WAKEUP => PTAT is sampled
}	


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void readElectricalOffset(void)
{
	unsigned int i,j;
//	unsigned int PTATUpperTEMP, PTATLowerTEMP;
	uint16_t bufferposition;
	uint16_t raw_value;
	
	HTPA32x32_SENSOR_readData(HTPA32x32_REGS_READ_OFFS_TOP_CMD,rxBuffer, 258);
	
//	PTATUpperTEMP = 256*rxBuffer[0] + rxBuffer[1];				// Gr om detta + BOT nedan. Anvnd stack
	
	bufferposition = 2; // +2 for skipping PTAT
	
	for(i = 0; i < 4; i++)	// 4 rows of electrical offset values
	{
		for(j = 0; j < PIX_PER_ROW; j++)
		{
			raw_value = 256*rxBuffer[bufferposition] + rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading TOP EO pos: %u, writing ElOffsetVij[%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition, i,i+4,i+8,i+12,j,i*32 + j, (i+4)*32 + j, (i+8)*32 + j, (i+12)*32 + j);
//			nrf_delay_ms(15);
			
			ElOffMatrixVij[i][j]= raw_value;	// Block 0
			ElOffMatrixVij[i + 4][j]= raw_value; // Block 1
			ElOffMatrixVij[i + 8][j]= raw_value; // Block 2
			ElOffMatrixVij[i + 12][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}
	
	
	HTPA32x32_SENSOR_readData(HTPA32x32_REGS_READ_OFFS_BOT_CMD,rxBuffer, 258);
	
//	PTATLowerTEMP = 256*rxBuffer[0] + rxBuffer[1];
	
	bufferposition = 2; // +2 for skipping PTAT
	
	for(i = 0; i < 4; i++)	// 4 rows of electrical offset values
	{
		for(j = 0; j < PIX_PER_ROW; j++)
		{
			raw_value = 256*rxBuffer[bufferposition] + rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading BOT EO pos: %u, writing ElOffsetVij[%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,31 - i,27 - i,23 - i,19 - i,j ,(31-i)*32 + j, (27-i)*32 + j, (23-i)*32 + j, (19-i)*32 + j);
//			nrf_delay_ms(15);
			
			ElOffMatrixVij[31 - i][j]= raw_value;	// Block 0
			ElOffMatrixVij[27 - i][j]= raw_value; // Block 1
			ElOffMatrixVij[23 - i][j]= raw_value; // Block 2
			ElOffMatrixVij[19 - i][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}	
	
}	



void HTPA32x32_init_sensor()
{	
//	printRawEEPROMData();
//	while(1);
	
	readEEPROMCalibrationData();	// Read all parameters from EEPROM located in sensor
	
#ifdef DebugEEPROM
	printEEPROMCalibrationData();
#endif
	
	HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_CONFIGURATION, 0x01);	// Wake sensor up, must be first command
//		HTPA32x32_read_DeviceID();	// Verifiera?
	
	// Write configuration to the sensor (using EEPROM (or default) data => Parameter = "0", OR "user" data (read from EEPROM) => Parameter = 1)
	InitMBITTRIM(0); 	// ADC Resolution
	InitBIASTRIM(0);	// ADC Bias current (TOP + BOT)
	InitCLKRIM(0);		// Measure Time
	InitBPATRIM(0);		// Common mode voltage for preamp (TOP + BOT)
	InitPUTRIM(0);		// I2C Pullup
	
	nrf_delay_ms(10);
	
	startSensorForElectricalOffsetReadout();
	waitForSensorReady();
	readElectricalOffset();

}

/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void readBlock(uint8_t blockNumber)
{
	uint16_t bufferposition;
	unsigned char i,j;
	
	unsigned char wr_i;;
	uint16_t raw_value;

	
	// Read TOP area [blocknumber]
	HTPA32x32_SENSOR_readData(HTPA32x32_REGS_READ_DATA_TOP_CMD,rxBuffer, 258); // 830ms

	PTATUpperBlock[blockNumber] = 256*rxBuffer[0] + rxBuffer[1];
	
	bufferposition = 2; // +2 for skipping PTAT
	
	for(i = 0; i<ROWS_PER_BLOCK; i++)
	{
		for(j = 0; j<PIX_PER_ROW; j++)
		{
			wr_i = blockNumber*ROWS_PER_BLOCK + i;
			raw_value = 256*rxBuffer[bufferposition] + rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB
			
//			printf("Reading Upper pos: %u, BLOCK %u,  writing Sensor[%u][%u] Pixel %u\n", bufferposition,blockNumber, wr_i,j,wr_i*32 + j);
//			nrf_delay_ms(15);
			
			SensorVij[wr_i][j]= raw_value;		
			
			bufferposition += 2;
		}
	}
	
	// Read BOT area [blocknumber], data order is reversed compared to upper section!
	HTPA32x32_SENSOR_readData(HTPA32x32_REGS_READ_DATA_BOT_CMD,rxBuffer, 258);

	PTATLowerBlock[blockNumber] = 256*rxBuffer[0] + rxBuffer[1];

	bufferposition = 2; // +2 for skipping PTAT
	
	for(i = 0; i<ROWS_PER_BLOCK; i++)
	{
		for(j = 0; j<PIX_PER_ROW; j++)
		{
			wr_i = 31 - blockNumber*ROWS_PER_BLOCK - i;
			raw_value = 256*rxBuffer[bufferposition] + rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading Lower pos: %u, BLOCK %u,  writing Sensor[%u][%u] Pixel %u\n", bufferposition,blockNumber, wr_i,j,wr_i*32 + j);
//			nrf_delay_ms(15);	
			
			SensorVij[wr_i][j]= raw_value;		
			
			bufferposition += 2;
		}
	}

}	


/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
unsigned int lookUpTemp(unsigned int TAmb, signed int dig, signed long PiC, unsigned int dontCalcTA){
	static unsigned int Tobject,y;
	signed int val;
	static signed long vx,vy,ydist;
	static unsigned int CurTACol;
	static signed int dTA;
	signed long long scale;


	//first check the position in x-axis of table
	if(!dontCalcTA)			// This is done only the first time we fun the calculaton, i.e. dontCalcTA = 0, use for example current pixel number as input
	{ 
		for(CurTACol = 0; CurTACol < NROFTAELEMENTS; CurTACol++)
		{
			if((XTATemps[CurTACol] <= TAmb) && (XTATemps[CurTACol+1] > TAmb))
					break;
			}
		dTA = TAmb - XTATemps[CurTACol];
	}

	if( (CurTACol >= NROFTAELEMENTS-1) )
	{
		//printf("XXXXXXXXXXXXXXXXXXXX lookUpTemp: CurTACol >= NROFTAELEMENTS-1) --- AMBIENT TEMPERATURE OUTSIDE TABLE RANGE --- XXXXXXXXXXXXXXXXXXXX\n");
		return 0;
	}

//	printf("Ta: %u dig: %d PiC: %ld ",TAmb, dig, PiC);
//	nrf_delay_ms(15);
	
	//now scale dig to PCSCALEVAL
	scale= (signed long long) PCSCALEVAL * (signed long long)dig;
	vx = (signed long) (scale / ( (signed long long) PiC) );


	val = vx + TABLEOFFSET;
	//now determine row
	y = val >> ADEXPBITS;
	

	ydist = (signed long) ADEQUIDISTANCE;
	
//	printf("TablePos: y = %u, TAcol = %u\n", y, CurTACol);
	
	if( y < (NROFADELEMENTS-1) )
	{	
		if(TempTable[y][CurTACol])
		{
			vx = ( ( ( (signed long)TempTable[y][CurTACol+1] - (signed long)TempTable[y][CurTACol] ) * (signed long)dTA)/(signed long)TAEQUIDISTANCE ) + (signed long)TempTable[y][CurTACol];
			vy = ( ( ( (signed long)TempTable[y+1][CurTACol+1] - (signed long)TempTable[y+1][CurTACol] ) * (signed long)dTA)/(signed long)TAEQUIDISTANCE ) + (signed long)TempTable[y+1][CurTACol];
				

			Tobject = (unsigned int)( (vy-vx) * ( (signed long)val - (signed long)YADValues[y] ) / ydist + (signed long)vx );
		}
		else
		{
			//printf("XXXXXXXXXXXXXXXXXXXX lookUpTemp:  TempTable[y][CurTACol]  --- TEMPERATURE TABLE EMPTY AV INDEX --- XXXXXXXXXXXXXXXXXXXX\n");
			return 0;
		}
	}
	else
	{
		//printf("XXXXXXXXXXXXXXXXXXXX lookUpTemp:  y < (NROFADELEMENTS-1) i.e. (%u < %u) --- Y-INDEX OUTSIDE TABLE RANGE --- XXXXXXXXXXXXXXXXXXXX\n",y,(NROFADELEMENTS-1));
		return 0;
	}

//	printf("Tobject %d\n",(unsigned short)Tobject);
//	nrf_delay_ms(10);
	return (unsigned short) Tobject;
}

/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void calcTemp(void)
{
	volatile unsigned long kl,ll,ml;
	//signed int temp;
	float temp;
	//static float pixelcompensation;
	
	uint16_t pixelposition;
	
	unsigned int i,j;
	
	signed long ThGradCompensation;
	signed long ThOffsetCompensation;
	signed long long VddCompensation;
	
	signed int Vij_comp;
	signed int Vij_compstar;
	signed int EOCompensation;
	signed int Vij_VDDcomp;
	
	unsigned long PowerGradScale;		

	signed short DeltaVdd;
	unsigned long long DividerVdd;
	unsigned long long DividerVdd2;
	
	PTATLong = PTATUpperBlock[0]+PTATUpperBlock[1]+PTATUpperBlock[2]+PTATUpperBlock[3]+PTATLowerBlock[0]+PTATLowerBlock[1]+PTATLowerBlock[2]+PTATLowerBlock[3];
	PTATLong = PTATLong / 8;	
	PTATLong = PTATLong&0x0000FFFF;
	
	Ta = PTATLong*PTATGrad + PTATOff + (float) 0.5;
	
		PowerGradScale = 1 << gradScale;				// OK
//		printf("PowerGradeScale= %lu", PowerGradeScale);

//	if(1)//SENSORREV1)
//	{
//		Vddlong=(unsigned long)VddSensorStack[0];
//		for(k=1;k<STACKSIZEVDD;k++)
//		{
//			Vddlong+=(unsigned long)VddSensorStack[k];
//		}
//		Vddlong/=(unsigned long)STACKSIZEVDD;
//		Voltage[outbuffer][VDDADDRESS]=(unsigned short)Vddlong;	//this overwrites the Vdd of the PCB with the Vdd of the sensor
//	}	

	DeltaVdd = 4000;// Example.... //(signed short)((signed long)Vddlong-(signed long)VddRef);
	DividerVdd = (unsigned long long)(1 << VddScalingGrad); //pow(2.0,(float)VddScaling));
	DividerVdd2 = (unsigned long long)(1 << VddScalingOff); //pow(2.0,(float)VddScalingOff));	

	for(i = 0; i < NUMBER_OF_ROWS; i++)
	{
		for(j = 0; j < PIX_PER_ROW; j++)
		{
			pixelposition = i * PIX_PER_ROW + j;

			// Thermal Gradient compensation
			ThGradCompensation = (signed long) ThGrad[i][j];
			ThGradCompensation *= (signed long) PTATLong; // was before Ta!!!
			ThGradCompensation /= (signed long) PowerGradScale;
								
			// Thermal Offset compensation
			ThOffsetCompensation = ThOff[i][j];

			Vij_comp = SensorVij[i][j] - ThGradCompensation - ThOffsetCompensation;
//			printf("i = %2u j = %2u Pixel = %4u\n", i, j, pixelposition);
//			printf("(Thermal compensated value) Vij_comp = Vij - ThGradCompensation - ThOffset = %4hu - %4ld - %4ld = %d\n", SensorVij[i][j], ThGradCompensation, ThOffsetCompensation, Vij_comp);
//			nrf_delay_ms(15);
					
			// Electrical Offset compensation
			//			FRN REFERENSKOD: EOCompensation = (signed long)(Voltage[outbuffer][k%ELAMOUNTHALF+ELOFFSET]<<(16-Resolution));
			EOCompensation = ElOffMatrixVij[i][j]; //elOffset[pixelposition%128];			// <<(16-Resolution) ???????? What?
					
			Vij_compstar = Vij_comp - EOCompensation;
//			printf("(Electrical offset compensated value) Vij_comp* = Vij_comp - elOffset[%u] = %u - %d = %d\n",(pixelposition%128) , Vij_comp, EOCompensation, Vij_compstar);
//			nrf_delay_ms(15);

					
			// VDD Compensation
			VddCompensation = (signed long long)VddCompGrad[i][j];
			VddCompensation *= (signed long long)PTATLong;
			VddCompensation /= (signed long long)DividerVdd;
			VddCompensation += (signed long long)VddCompOff[i][j];
			VddCompensation *= (signed long long)DeltaVdd;
			VddCompensation /= (signed long long)DividerVdd2;
//		VoltageLong += (signed long)VddCompensation;
					
			Vij_VDDcomp = Vij_compstar- (signed int) VddCompensation;
   		//printf("(VDD offset compensated value) Vij_VDDcomp* = Vij_comp* - VddCompensation = %d - %d = %d\n", Vij_compstar, (signed int) VddCompensation, Vij_VDDcomp);
//			nrf_delay_ms(15);
					
//					for(j=1;j<StackSize;j++)
//					{
//						aveloff+=eloffstack[k%ELAMOUNTHALF][j];		//eloffstack needs to be filled from time to time with the blind pixels
//					}
//					
//					VoltageLong-=(signed long)(aveloff/(unsigned long)StackSize);				
//					VoltageSigned=(signed short)Voltage[outbuffer][k]-(signed short)VoltageLong;

//				pixelcompensation = (float)PCSCALEVAL/PixC[pixelposition];
//				printf("Pixelcompensation : %.2f ",pixelcompensation);
//			
//				compensatedpixel = (signed int) Vij_compstar * pixelcompensation;
//				printf("Compensated pixel value : %4d ", compensatedpixel);
			
			// Fyll debugtabeller!
			TEMP_ThermalGradCompTable[i][j] = ThGradCompensation;
			TEMP_ThermalOffsCompTable[i][j] = ThOffsetCompensation;
			TEMP_VddCompTable[i][j] = VddCompensation;
			TEMP_IndataToLookupTable[i][j] = Vij_VDDcomp;

			temp = lookUpTemp(Ta, Vij_VDDcomp, PixC[i][j], pixelposition);		// Vij_VDDcomp, inte compensatedpixel!!! Kompensering grs i funktionen!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			TaSensor = Ta/10-273;
			//resultatet r i dK, konvertera till grader C
			temp = temp / 10;
			temp = temp - 273;
			SensorImage[i][j] = temp;
			
//			printf("Final temperature pixel value : %4d\n", temp);
						
		}
	}
}



/********************************************************************
 * Function:		  
 *
 * Description:	  
 *
 * Dependencies:	 
 *							
 *******************************************************************/
void HTPA32x32_stopSensor()
{
		/*nrf_delay_ms(20);
	nrf_delay_ms(20);
	nrf_delay_ms(20);
	nrf_delay_ms(20);
	nrf_delay_ms(20);
	nrf_delay_ms(20);
		waitForSensorReady();*/
		HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_CONFIGURATION, 0x00);		// [1 0 0 0]
		nrf_delay_ms(20);
		waitForSensorReady();
		HTPA32x32_SENSOR_writeConfig(HTPA32x32_REGS_CONFIGURATION, 0x00);
}
void HTPA32x32_readBlockInSensor(uint8_t block_to_read)
{
	PMAIF_StartI2C();
	startSensor(block_to_read);
	nrf_delay_ms(20);
	waitForSensorReady();
	readBlock(block_to_read);
	
	calcTemp();
	if(NrOfDefPix!=0)
		compensateForDefPix();
	
	PMAIF_StopI2C();
}
void HTPA32x32_readCompleteSensor()
{
	PMAIF_StartI2C();
	startSensor(0);
	nrf_delay_ms(20);
	waitForSensorReady();
	readBlock(0);
	
	startSensor(1);
	nrf_delay_ms(20);
	waitForSensorReady();
	readBlock(1);
	
	startSensor(2);
	nrf_delay_ms(20);
	waitForSensorReady();
	readBlock(2);
	
	startSensor(3);
	nrf_delay_ms(20);
	waitForSensorReady();
	readBlock(3);

	calcTemp();

	if(NrOfDefPix!=0)
		compensateForDefPix();
  // Check sensor matrix order
//	for (uint8_t i = 0; i < 32; i++)
//		for (uint8_t j = 0; j < 32; j++)
//			SensorImage[i][j] = (unsigned short) (i* 32 + j);

#ifdef DebugSensordata		
	printSensorData();
#endif	
	PMAIF_StopI2C();
}	
void compensateForDefPix()
{
	int row_t = 0;
	int col_t = 0;
	
	for(int i=0;i<NrOfDefPix;i++)
	{

			row_t = deadPix[i].addressOfDefPix / 32;
			col_t = deadPix[i].addressOfDefPix - (row_t*32);
			if(row_t > 15)		//Bottom half
				row_t = 16+(31-row_t);
			
			
			SensorImage[row_t][col_t] = SensorImage[row_t-1][col_t-1]*deadPix[i].deadMaskPixUpperLeft+SensorImage[row_t-1][col_t]*deadPix[i].deadMaskPixUpper+SensorImage[row_t-1][col_t+1]*deadPix[i].deadMaskPixUpperRight+SensorImage[row_t][col_t-1]*deadPix[i].deadMaskPixLeft+SensorImage[row_t][col_t+1]*deadPix[i].deadMaskPixRight+SensorImage[row_t+1][col_t-1]*deadPix[i].deadMaskPixLowerLeft+SensorImage[row_t+1][col_t]*deadPix[i].deadMaskPixLower+SensorImage[row_t+1][col_t+1]*deadPix[i].deadMaskPixLowerRight;
			SensorImage[row_t][col_t] =SensorImage[row_t][col_t] / deadPix[i].numberOfMasks;
			
	}
}

/********************************************************************
 * Function:		  
 *
 * Description:	  
 * Updates thermal offsets and other sensordata. 
 * Dependencies:	 
 *							
 *******************************************************************/
void HTPA32x32_uppdate_sensor_data()
{
	
	PMAIF_StartI2C();
	
	int i;
	int j;
	int readdataposition;
	int pixelposition_i;
	uint16_t raw_value;
	uint16_t bufferposition;
	//now load the Thermal-Grad and -Offset, Observe data order (both arrays) in datasheet chapter 8.0. Word order 0..511, 1023..512 is unclear! Indications suggest order is reversed in middle of EEPROM data!
// Total 2032 bytes = 1024 x 16-bit signed values per matrix
	for(i = 0; i < NUMBER_OF_ROWS; i++)
	{
		for(j = 0; j < PIX_PER_ROW; j++)
		{
			readdataposition = HTPA32x32_REGM_Th1 + (i * PIX_PER_ROW + j) * 2;
			
//			printf("Reading ThGrad[%u][%u] (Memory address: 0x%04X) - ", i, j, readdataposition);
//			nrf_delay_ms(15);
			
			HTPA32x32_MEMORY_readData(readdataposition,rxBuffer,sizeof(char)*2);

			if(i < 16)	// Top half
			{
				
//				pixelposition = i * PIX_PER_ROW + j;
//				printf("Writing ThGrad[%u][%u] (Pixel %u)\n",i,j,pixelposition);
//				nrf_delay_ms(15);
				
				ThGrad[i][j]= rxBuffer[0] + 256*rxBuffer[1];
			}
			else	// Bot half
			{
				pixelposition_i = 47 - i;
				
//				pixelposition = pixelposition_i * PIX_PER_ROW + j;
//				printf("Writing ThGrad[%u][%u] (Pixel %u)\n",pixelposition_i,j,pixelposition);
//				nrf_delay_ms(15);
				
				ThGrad[pixelposition_i][j]= rxBuffer[0] + 256*rxBuffer[1];
			}
			
			
			readdataposition = HTPA32x32_REGM_Th2 + (i * PIX_PER_ROW + j) * 2;
			
//			printf("Reading ThOff[%u][%u] (Memory address: 0x%04X) - ", i, j, readdataposition);
//			nrf_delay_ms(15);
			
			HTPA32x32_MEMORY_readData(readdataposition,rxBuffer,sizeof(char)*2);
			
			if(i < 16)	// Top half
			{
				
//				pixelposition = i * PIX_PER_ROW + j;
//				printf("Writing ThOff[%u][%u] (Pixel %u)\n",i,j,pixelposition);
//				nrf_delay_ms(15);
				
				ThOff[i][j]= rxBuffer[0] + 256*rxBuffer[1];
			}
			else	// Bot half
			{
				pixelposition_i = 47 - i;
				
//				pixelposition = pixelposition_i * PIX_PER_ROW + j;
//				printf("Writing ThOff[%u][%u] (Pixel %u)\n",pixelposition_i,j,pixelposition);
//				nrf_delay_ms(15);
				
				ThOff[pixelposition_i][j]= rxBuffer[0] + 256*rxBuffer[1];
			}
		}
	}

//Read VddCompGrad and save it in VddCompGrad-matrix
	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_VddCompGrad,rxBuffer, 512);
	
	bufferposition = 0;
	
	for(i = 0; i < 4; i++)	// First 4 rows are in TOP -> BOT order 
	{
		for(j = 0; j < PIX_PER_ROW; j++) 
		{
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading VddCompGrad pos: %u ([%u][%u]), writing VddCompGrad%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,i,j, i,i+4,i+8,i+12,j,i*32 + j, (i+4)*32 + j, (i+8)*32 + j, (i+12)*32 + j);
//			nrf_delay_ms(15);
			
			VddCompGrad[i][j]= raw_value;	// Block 0 
			VddCompGrad[i + 4][j]= raw_value; // Block 1
			VddCompGrad[i + 8][j]= raw_value; // Block 2
			VddCompGrad[i + 12][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}

	// Keep bufferposition and continue in reverse order
	
	for(i = 0; i < 4; i++)	// Second 4 rows are in BOT -> TOP order 
	{
		for(j = 0; j < PIX_PER_ROW; j++) 
		{
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading VddCompGrad pos: %u ([%u][%u]), writing VddCompGrad[%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,i+4,j,31 - i,27 - i,23 - i,19 - i,j ,(31-i)*32 + j, (27-i)*32 + j, (23-i)*32 + j, (19-i)*32 + j);
//			nrf_delay_ms(15);
			
			VddCompGrad[31 - i][j]= raw_value;	// Block 0
			VddCompGrad[27 - i][j]= raw_value; // Block 1
			VddCompGrad[23 - i][j]= raw_value; // Block 2
			VddCompGrad[19 - i][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}



//Read VddCompOff and save it in VddCompOff-matrix
HTPA32x32_MEMORY_readData(HTPA32x32_REGM_VddCompOff,rxBuffer, 512);

	bufferposition = 0;
	for(i = 0; i < 4; i++)	// First 4 rows are in TOP -> BOT order 
	{
		for(j = 0; j < PIX_PER_ROW; j++) 
		{
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading VddCompOff pos: %u ([%u][%u]), writing VddCompOff[%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,i,j, i,i+4,i+8,i+12,j,i*32 + j, (i+4)*32 + j, (i+8)*32 + j, (i+12)*32 + j);
//			nrf_delay_ms(15);
			
			VddCompOff[i][j]= raw_value;	// Block 0 
			VddCompOff[i + 4][j]= raw_value; // Block 1
			VddCompOff[i + 8][j]= raw_value; // Block 2
			VddCompOff[i + 12][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}

	// Keep bufferposition and continue in reverse order
	
	for(i = 0; i < 4; i++)	// Second 4 rows are in BOT -> TOP order 
	{
		for(j = 0; j < PIX_PER_ROW; j++) 
		{
			raw_value = rxBuffer[bufferposition] + 256*rxBuffer[bufferposition + 1];			// nRF52852 = Little endian = LSB, MSB			Sensor = Big Endian = MSB, LSB

//			printf("Reading VddCompOff pos: %u ([%u][%u]), writing VddCompOff[%u, %u, %u, %u][%u] Pixel %u, %u, %u, %u\n", bufferposition,i+4,j,31 - i,27 - i,23 - i,19 - i,j ,(31-i)*32 + j, (27-i)*32 + j, (23-i)*32 + j, (19-i)*32 + j);
//			nrf_delay_ms(15);
			
			VddCompOff[31 - i][j]= raw_value;	// Block 0
			VddCompOff[27 - i][j]= raw_value; // Block 1
			VddCompOff[23 - i][j]= raw_value; // Block 2
			VddCompOff[19 - i][j]= raw_value; // Block 3
			
			bufferposition += 2;
		}
	}
//************************************************************************************************************************************
	nrf_delay_ms(10);
	PMAIF_StopI2C();
}
bool HTPA32x32_verify_device(void)
{
	DeviceID = 0;
	// Read DeviceID
	HTPA32x32_MEMORY_readData(HTPA32x32_REGM_DevID,rxBuffer, sizeof(long));		// 4 bytes
	DeviceID = rxBuffer[0] + 256*rxBuffer[1] + 256*256*rxBuffer[2] + 256*256*256*rxBuffer[3];	

	if(DeviceID == 0xA0)
		return true;
	else
		return false;
	
}

