/**
 * Copyright (c) 2015 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef HTPA32x32_H__
#define HTPA32x32_H__

#include <stdint.h>
#include "app_twi.h"
#include "globals.h"
  

//    #ifdef HTPA32x32dL2_1HiSiF5_0_Gain3k3
#define TABLENUMBER		114
#define PCSCALEVAL		100000000 //327000000000		//PixelConst scale value for table... lower 'L' for (long)
#define NROFTAELEMENTS 	7
#define NROFADELEMENTS 	1595	//130 possible due to Program memory, higher values possible if NROFTAELEMENTS is decreased
#define TAEQUIDISTANCE	100		//dK
#define ADEQUIDISTANCE	64		//dig
#define ADEXPBITS		6		//2^ADEXPBITS=ADEQUIDISTANCE
#define TABLEOFFSET		1024
//		#define EQUIADTABLE		//if defined, ADELEMENTS have to be 2^N quantizied! else more CPU Power is needed
//		#ifdef EQUIADTABLE
//			#undef FLOATTABLE
//		#endif   
#define HTPA32x32_MBITTRIMDefault 0x2C	//use REF_CAL=2 here. Table does not match, so GlobalGain ist set to 50 % to compensate this.
//		#define SensRv 1				//Sensor Revision is set to 1 (Redesign)
//    #endif	


#define HTPA32x32_BIAScurrentDefault		0x05
#define HTPA32x32_CLKTRIMDefault				0x15 //0x20 to let it run with 10 Hz
#define HTPA32x32_BPATRIMDefault				0x0C
// #define HTPA32x32_MBITTRIMDefault				0x0C
#define HTPA32x32_PUTRIMDefault					0x88

#ifdef __cplusplus
extern "C" {
#endif

	

#define ROWS_PER_BLOCK 4
	
// #define ELOFFSET 1024			//start address of el. Offset
#define ELAMOUNT 256
#define ELAMOUNTHALF 128
#define StackSize 16			//must be choosen by the user!	
	
// 0x3A / 0x3B is the HTPA32x32's address, it contains
// R/W bit and "nrf_drv_twi" requires slave address without this bit, hence shifting.
#define HTPA32x32_SENSOR_ADDR					  (0x34U >> 1)		//  = 0x1AU

#define HTPA32x32_REGS_CONFIGURATION			0x01
#define HTPA32x32_REGS_STATUS						0x02
#define HTPA32x32_REGS_TRIM_1						0x03
#define HTPA32x32_REGS_TRIM_2						0x04
#define HTPA32x32_REGS_TRIM_3						0x05
#define HTPA32x32_REGS_TRIM_4						0x06
#define HTPA32x32_REGS_TRIM_5						0x07
#define HTPA32x32_REGS_TRIM_6						0x08
#define HTPA32x32_REGS_TRIM_7						0x09
#define HTPA32x32_REGS_READ_DATA_TOP_CMD	0x0A
#define HTPA32x32_REGS_READ_DATA_BOT_CMD	0x0B
#define HTPA32x32_REGS_READ_OFFS_TOP_CMD	0x0A
#define HTPA32x32_REGS_READ_OFFS_BOT_CMD	0x0B

//#define HTPA32x32_ADDR_MEMORY					  (0xA0U >> 1)		//  = 0x50U
#define HTPA32x32_MEMORY_ADDR					  (0xA0U >> 1)		//  = 0x50U
		
#define HTPA32x32_REGM_PixCMin					0x00	// 4 bytes Float
#define HTPA32x32_REGM_PixCMax					0x04	// 4 bytes Float
#define HTPA32x32_REGM_GradScale				0x08	// 3 bytes
#define HTPA32x32_REGM_TableNumber			0x0B	//  bytes
#define HTPA32x32_REGM_Epsilon					0x0D	
#define HTPA32x32_REGM_MBITCal					0x1A	//  bytes
#define HTPA32x32_REGM_BIASCal					0x1B	//  bytes
#define HTPA32x32_REGM_CLKCal						0x1C	//  bytes
#define HTPA32x32_REGM_BPACal						0x1D	//  bytes
#define HTPA32x32_REGM_PUCal						0x1E	//  bytes
#define HTPA32x32_REGM_VddCalib		 			0x26	// Kallades VddMeasTh1 i tidigare datablad
#define HTPA32x32_REGM_PTATGrad					0x34	// 4 bytes
#define HTPA32x32_REGM_PTATOffs					0x38	// 4 bytes
#define AdrVddTh1 0x46	// Finns inte med i senaste datablad (rev 17)!!!

#define HTPA32x32_REGM_VddScalingGrad		0x4E	// VddScGrad
#define HTPA32x32_REGM_VddScalingOff		0x4F	// VddScOff
#define HTPA32x32_REGM_GlobalOffset			0x54	// 1 byte
#define HTPA32x32_REGM_GlobalGain				0x55	// 2 bytes
#define HTPA32x32_REGM_MBITUser					0x60	//  bytes
#define HTPA32x32_REGM_BIASUser					0x61	//  bytes
#define HTPA32x32_REGM_CLKUser					0x62	//  bytes
#define HTPA32x32_REGM_BPAUser					0x63	//  bytes
#define HTPA32x32_REGM_PUUser						0x64	//  bytes
#define HTPA32x32_REGM_DevID						0x74	//  bytes
#define HTPA32x32_REGM_NrOfDefPix				0x7F	// 1 byte
#define HTPA32x32_REGM_DefPixAddr				0x80  // bytes
#define HTPA32x32_REGM_DefPixMask				0xB0	// bytes

#define HTPA32x32_REGM_VddCompGrad			0x340 // VddCompGrad 16-bits signed array
#define HTPA32x32_REGM_VddCompOff				0x540 // VddCompOff 16-bits signed array
#define HTPA32x32_REGM_Th1							0x740	//  bytes
#define HTPA32x32_REGM_Th2							0xF40	//  bytes
#define HTPA32x32_REGM_PixC							0x1740	//  bytes

#define HTPA32x32_MEMORY_READ(p_reg_addr, p_buffer, byte_cnt) \
    APP_TWI_WRITE(HTPA32x32_MEMORY_ADDR, p_reg_addr, 2,        APP_TWI_NO_STOP), \
    APP_TWI_READ (HTPA32x32_MEMORY_ADDR, p_buffer,   byte_cnt, 0)

#define HTPA32x32_SENSOR_READ(p_reg_addr, p_buffer, byte_cnt) \
    APP_TWI_WRITE(HTPA32x32_SENSOR_ADDR, p_reg_addr, 1,        APP_TWI_NO_STOP), \
    APP_TWI_READ (HTPA32x32_SENSOR_ADDR, p_buffer,   byte_cnt, 0)

#define HTPA32x32_SENSOR_READ_NOSTOP(p_reg_addr, p_buffer, byte_cnt) \
    APP_TWI_WRITE(HTPA32x32_SENSOR_ADDR, p_reg_addr, 1,        APP_TWI_NO_STOP), \
    APP_TWI_READ (HTPA32x32_SENSOR_ADDR, p_buffer,   3, APP_TWI_NO_STOP), \
		APP_TWI_READ (HTPA32x32_SENSOR_ADDR, p_buffer,   3, 0)
		
#define HTPA32x32_SENSOR_INIT_TRANSFER_COUNT 2
extern app_twi_transfer_t HTPA32x32_init_deviceX[HTPA32x32_SENSOR_INIT_TRANSFER_COUNT];

void HTPA32x32_init_sensor(void);
//bool HTPA32x32_verify_device(nrf_drv_twi_t const *p_instance);
uint32_t HTPA32x32_read_DeviceID(void);
void HTPA32x32_readCompleteSensor(void);
void HTPA32x32_readBlockInSensor(uint8_t block_to_read);
void compensateForDefPix(void);
void HTPA32x32_uppdate_sensor_data(void);
void HTPA32x32_stopSensor(void);
bool HTPA32x32_verify_device(void);


#ifdef __cplusplus
}
#endif

#endif // HTPA32x32_H__
